"""Tests for the tulip.synth module when gr1c is used."""
import logging
from synth_test import multiple_env_actions_check


logging.getLogger('tulip').setLevel(logging.ERROR)
logging.getLogger('tulip.interfaces.gr1c').setLevel(logging.DEBUG)


def multiple_env_actions_test():
    multiple_env_actions_check('gr1c')



#!/usr/bin/env python
"""Example of direct GR(1) specification, using only boolean variables.

This example illustrates the use of TuLiP to synthesize a reactive
controller for a GR(1) specification.  We code the specification
directly in GR(1) form and then use TuLiP to synthesize a reactive
controller.

The system is modeled as a discrete transition system in which the
robot can be located anyplace on a 2x3 grid of cells:

    +----+----+----+
    | X3 | X4 | X5 |
    +----+----+----+
    | X0 | X1 | X2 |
    +----+----+----+

The robot is allowed to transition between any two adjacent cells;
diagonal motions are not allowed.  The robot should continuously
revisit the cell X5.

The environment consists of a single state called 'park' that
indicates that the robot should move to cell X0.

The system specification in its simplest form is given by

  []<>park -> []<>X5 && [](park -> <>X0)

We must convert this specification into GR(1) form:

  env_init && []env_safe && []<>env_prog_1 && ... && []<>env_prog_m ->
      sys_init && []sys_safe && []<>sys_prog_1 && ... && []<>sys_prog_n
"""
# 21 Jul 2013, Richard M. Murray (murray@cds.caltech.edu)
# Import the packages that we need
import logging

from tulip import spec
from tulip import synth
from tulip.transys import machines


logging.basicConfig(level=logging.WARNING)


#
# Environment specification
#
# The environment can issue a park signal that the robot must respond
# to by moving to the lower left corner of the grid.  We assume that
# the park signal is turned off infinitely often.
#
env_vars = {'park'}
env_init = set()                # empty set
env_safe = set()                # empty set
env_prog = '!park'              # []<>(!park)

#
# System dynamics
#
# The system specification describes how the system is allowed to move
# and what the system is required to do in response to an environmental
# action.
#
sys_vars = {'X0', 'X1', 'X2', 'X3', 'X4', 'X5'}
sys_init = {'X0'}
sys_safe = {
    'X0 -> X (X1 || X3)',
    'X1 -> X (X0 || X4 || X2)',
    'X2 -> X (X1 || X5)',
    'X3 -> X (X0 || X4)',
    'X4 -> X (X3 || X1 || X5)',
    'X5 -> X (X4 || X2)',
}

sys_safe.add(synth.exactly_one({'X0', 'X1', 'X2', 'X3', 'X4', 'X5'})[0])

sys_prog = set()                # empty set

#
# System specification
#
# The system specification is that the robot should repeatedly revisit
# the upper right corner of the grid while at the same time responding
# to the park signal by visiting the lower left corner.  The LTL
# specification is given by
#
#     []<> X5 && [](park -> <>X0)
#
# Since this specification is not in GR(1) form, we introduce an
# environment variable X0reach that is initialized to True and the
# specification [](park -> <>X0) becomes
#
#     [](X (X0reach) <-> X0 || (X0reach && !park))
#

# Augment the system description to make it GR(1)
sys_vars |= {'X0reach'}
sys_init |= {'X0reach'}
sys_safe |= {'(X (X0reach) <-> X0) || (X0reach && !park)'}
sys_prog |= {'X0reach', 'X5'}

# Create a GR(1) specification
specs = spec.GRSpec(env_vars, sys_vars, env_init, sys_init,
                    env_safe, sys_safe, env_prog, sys_prog)

#
# Controller synthesis
#
# The controller decides based on current variable values only,
# without knowing yet the next values that environment variables take.
# A controller with this information flow is known as Moore.
specs.moore = True
# Ask the synthesizer to find initial values for system variables
# that, for each initial values that environment variables can
# take and satisfy `env_init`, the initial state satisfies
# `env_init /\ sys_init`.
specs.qinit = '\E \A'  # i.e., "there exist sys_vars: forall sys_vars"

# At this point we can synthesize the controller
# using one of the available methods.
strategy = synth.synthesize('omega', specs)
assert strategy is not None, 'unrealizable'

# Generate a graphical representation of the controller for viewing,
# or a textual representation if pydot is missing.
if not strategy.save('gr1.png'):
    print(strategy)

# simulate
print(strategy)
machines.random_run(strategy, N=10)



# Copyright (c) 2011-2015 by California Institute of Technology
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the California Institute of Technology nor
#    the names of its contributors may be used to endorse or promote
#    products derived from this software without specific prior
#    written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL CALTECH
# OR THE CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
# USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
# OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
"""
Interface to gr1c

  - U{http://scottman.net/2012/gr1c}
  - release documentation at U{https://tulip-control.github.io/gr1c/}

In general, functions defined here will raise CalledProcessError (from
the subprocess module) or OSError if an exception occurs while
interacting with the gr1c executable.

Use the logging module to throttle verbosity.
"""
from distutils.version import StrictVersion
import logging
import copy
import os
import subprocess
import tempfile
import json
import xml.etree.ElementTree as ET
import networkx as nx
from tulip.spec import GRSpec, translate


GR1C_MIN_VERSION = '0.9.0'
GR1C_BIN_PREFIX = ""
DEFAULT_NAMESPACE = "http://tulip-control.sourceforge.net/ns/1"
_hl = 60 * '-'
logger = logging.getLogger(__name__)


def check_gr1c():
    """Return `True` if `gr1c >= require_version` found in PATH."""
    try:
        v = subprocess.check_output(["gr1c", "-V"])
    except OSError:
        return False
    v = v.split()[1]
    if StrictVersion(v) >= StrictVersion(GR1C_MIN_VERSION):
        return True
    return False


def _assert_gr1c():
    """Raise `Exception` if `gr1c` not in PATH."""
    if check_gr1c():
        return
    raise Exception(
        '`gr1c >= {v}` not found in the PATH.\n'.format(v=GR1C_MIN_VERSION) +
        'Unless an alternative synthesis tool is installed,\n'
        'it will not be possible to realize GR(1) specifications.\n'
        'Consult installation instructions for gr1c at:\n'
        '\t http://scottman.net/2012/gr1c\n'
        'or the TuLiP User\'s Guide about alternatives.')


def get_version():
    """Get version of gr1c as detected by TuLiP.

    Failure to find the gr1c program or errors in parsing the received
    version string will cause an exception.

    @return: (major, minor, micro), a tuple of int
    """
    try:
        v_str = subprocess.check_output(["gr1c", "-V"])
    except OSError:
        raise OSError('gr1c not found')
    v_str = v_str.split()[1]
    try:
        major, minor, micro = v_str.split(".")
        major = int(major)
        minor = int(minor)
        micro = int(micro)
    except ValueError:
        raise ValueError('gr1c version string is not recognized: '+str(v_str))
    return (major, minor, micro)

def _untaglist(x, cast_f=float,
               namespace=DEFAULT_NAMESPACE):
    """Extract list from given tulipcon XML tag (string).

    Use function cast_f for type-casting extracting element strings.
    The default is float, but another common case is cast_f=int (for
    "integer").  If cast_f is set to None, then items are left as
    extracted, i.e. as strings.

    The argument x can also be an instance of
    xml.etree.ElementTree._ElementInterface ; this is mainly for
    internal use, e.g. by the function untagpolytope and some
    load/dumpXML methods elsewhere.

    Return result as 2-tuple, containing name of the tag (as a string)
    and the list obtained from it.
    """
    if not isinstance(x, str) and not isinstance(x, ET._ElementInterface):
        raise TypeError("tag to be parsed must be given as" +
            " a string or ElementTree._ElementInterface.")

    if isinstance(x, str):
        elem = ET.fromstring(x)
    else:
        elem = x

    if (namespace is None) or (len(namespace) == 0):
        ns_prefix = ""
    else:
        ns_prefix = "{"+namespace+"}"

    # Extract list
    if cast_f is None:
        cast_f = str
    litems = elem.findall(ns_prefix+'litem')
    if len(litems) > 0:
        li = [cast_f(k.attrib['value']) for k in litems]
    elif elem.text is None:
        li = []
    else:
        li = [cast_f(k) for k in elem.text.split()]

    return (elem.tag, li)

def _untagdict(x, cast_f_keys=None, cast_f_values=None,
               namespace=DEFAULT_NAMESPACE, get_order=False):
    """Extract dictionary from given tulipcon XML tag (string).

    Use functions cast_f_keys and cast_f_values for type-casting
    extracting key and value strings, respectively, or None.  The
    default is None, which means the extracted keys (resp., values)
    are left untouched (as strings), but another common case is
    cast_f_values=int (for "integer") or cast_f_values=float (for
    "floating-point numbers"), while leaving cast_f_keys=None to
    indicate dictionary keys are strings.

    The argument x can also be an instance of
    xml.etree.ElementTree._ElementInterface ; this is mainly for
    internal use, e.g. by the function untagpolytope and some
    load/dumpXML methods elsewhere.

    Return result as 2-tuple, containing name of the tag (as a string)
    and the dictionary obtained from it.  If get_order is True, then
    return a triple, where the first two elements are as usual and the
    third is the list of keys in the order they were found.
    """
    if not isinstance(x, str) and not isinstance(x, ET._ElementInterface):
        raise TypeError("tag to be parsed must be given " +
            "as a string or ElementTree._ElementInterface.")

    if isinstance(x, str):
        elem = ET.fromstring(x)
    else:
        elem = x

    if (namespace is None) or (len(namespace) == 0):
        ns_prefix = ""
    else:
        ns_prefix = "{"+namespace+"}"

    # Extract dictionary
    items_li = elem.findall(ns_prefix+"item")
    if cast_f_keys is None:
        cast_f_keys = str
    if cast_f_values is None:
        cast_f_values = str
    di = dict()
    if get_order:
        key_list = []
    for item in items_li:
        # N.B., we will overwrite duplicate keys without warning!
        di[cast_f_keys(item.attrib["key"])] = cast_f_values(item.attrib["value"])
        if get_order:
            key_list.append(item.attrib["key"])
    if get_order:
        return (elem.tag, di, key_list)
    else:
        return (elem.tag, di)

def load_aut_xml(x, namespace=DEFAULT_NAMESPACE):
    """Return strategy constructed from output of gr1c.

    @param x: a string or an instance of
        xml.etree.ElementTree._ElementInterface

    @type spec0: L{GRSpec}
    @param spec0: GR(1) specification with which to interpret the
        output of gr1c while constructing a MealyMachine, or None if
        the output from gr1c should be used as is.  Note that spec0
        may differ from the specification in the given tulipcon XML
        string x.  If you are unsure what to do, try setting spec0 to
        whatever L{gr1cint.synthesize} was invoked with.

    @return: if a strategy is given in the XML string, return it as
        C{networkx.DiGraph}. Else, return (L{GRSpec}, C{None}), where
        the first element is the specification as read from the XML string.
    """
    if not isinstance(x, str) and not isinstance(x, ET._ElementInterface):
        raise TypeError("tag to be parsed must be given " +
            "as a string or ElementTree._ElementInterface.")

    if isinstance(x, str):
        elem = ET.fromstring(x)
    else:
        elem = x

    if (namespace is None) or (len(namespace) == 0):
        ns_prefix = ""
    else:
        ns_prefix = "{"+namespace+"}"

    if elem.tag != ns_prefix+"tulipcon":
        raise TypeError("root tag should be tulipcon.")
    if ("version" not in elem.attrib.keys()):
        raise ValueError("unversioned tulipcon XML string.")
    if int(elem.attrib["version"]) != 1:
        raise ValueError("unsupported tulipcon XML version: "+
            str(elem.attrib["version"]))

    # Extract discrete variables and LTL specification
    (tag_name, env_vardict, env_vars) = _untagdict(elem.find(
        ns_prefix+"env_vars"), get_order=True)
    (tag_name, sys_vardict, sys_vars) = _untagdict(elem.find(
        ns_prefix+"sys_vars"), get_order=True)

    env_vars = _parse_vars(env_vars, env_vardict)
    sys_vars = _parse_vars(sys_vars, sys_vardict)

    s_elem = elem.find(ns_prefix+"spec")
    spec = GRSpec(env_vars=env_vars, sys_vars=sys_vars)
    for spec_tag in ["env_init", "env_safety", "env_prog",
                     "sys_init", "sys_safety", "sys_prog"]:
        if s_elem.find(ns_prefix+spec_tag) is None:
            raise ValueError("invalid specification in tulipcon XML string.")
        (tag_name, li) = _untaglist(s_elem.find(ns_prefix+spec_tag),
                                    cast_f=str, namespace=namespace)
        li = [v.replace("&lt;", "<") for v in li]
        li = [v.replace("&gt;", ">") for v in li]
        li = [v.replace("&amp;", "&") for v in li]
        setattr(spec, spec_tag, li)

    aut_elem = elem.find(ns_prefix+"aut")
    if aut_elem is None or (
        (aut_elem.text is None) and len(aut_elem.getchildren()) == 0):
        mach = None
        return (spec, mach)

    # Assume version 1 of tulipcon XML
    if aut_elem.attrib["type"] != "basic":
        raise ValueError("Automaton class only recognizes type \"basic\".")
    node_list = aut_elem.findall(ns_prefix+"node")
    id_list = []  # For more convenient searching, and to catch redundancy
    A = nx.DiGraph()
    A.env_vars = env_vars
    A.sys_vars = sys_vars
    for node in node_list:
        this_id = int(node.find(ns_prefix+"id").text)
        #this_name = node.find(ns_prefix+"anno").text  # Assume version 1
        (tag_name, this_name_list) = _untaglist(node.find(ns_prefix+"anno"),
                                                cast_f=int)
        if len(this_name_list) == 2:
            (mode, rgrad) = this_name_list
        else:
            (mode, rgrad) = (-1, -1)
        (tag_name, this_child_list) = _untaglist(
            node.find(ns_prefix+"child_list"),
            cast_f=int
        )
        if tag_name != ns_prefix+"child_list":
            # This really should never happen and may not even be
            # worth checking.
            raise ValueError("failure of consistency check " +
                "while processing aut XML string.")
        (tag_name, this_state) = _untagdict(node.find(ns_prefix+"state"),
                                            cast_f_values=int,
                                            namespace=namespace)

        if tag_name != ns_prefix+"state":
            raise ValueError("failure of consistency check " +
                "while processing aut XML string.")
        if this_id in id_list:
            logger.warn("duplicate nodes found: "+str(this_id)+"; ignoring...")
            continue
        id_list.append(this_id)

        logger.info('loaded from gr1c result:\n\t' +str(this_state) )

        A.add_node(this_id, state=copy.copy(this_state),
                   mode=mode, rgrad=rgrad)
        for next_node in this_child_list:
            A.add_edge(this_id, next_node)
    return A

def _parse_vars(variables, vardict):
    """Helper for parsing env, sys variables.
    """
    domains = []
    for v in variables:
        dom = vardict[v]
        if dom[0] == "[":
            end_ind = dom.find("]")
            if end_ind < 0:
                raise ValueError("invalid domain for variable \""+
                    str(v)+"\": "+str(dom))
            dom_parts = dom[1:end_ind].split(",")
            if len(dom_parts) != 2:
                raise ValueError("invalid domain for variable \""+
                    str(v)+"\": "+str(dom))
            domains.append((int(dom_parts[0]), int(dom_parts[1])))
        elif dom == "boolean":
            domains.append("boolean")
        else:
            raise ValueError("unrecognized type of domain for variable \""+
                str(v)+"\": "+str(dom))
    variables = dict([
        (v, domains[i])
        for i, v in enumerate(variables)
    ])
    return variables

def load_aut_json(x):
    """Return strategy constructed from output of gr1c

    @param x: string or file-like object

    @return: strategy as C{networkx.DiGraph}, like the return value of
        L{load_aut_xml}
    """
    try:
        autjs = json.loads(x)
    except TypeError:
        autjs = json.load(x)
    if autjs['version'] != 1:
        raise ValueError('Only gr1c JSON format version 1 is supported.')
    # convert to nx
    A = nx.DiGraph()
    symtab = autjs['ENV'] + autjs['SYS']
    A.env_vars = dict([v.items()[0] for v in autjs['ENV']])
    A.sys_vars = dict([v.items()[0] for v in autjs['SYS']])
    omit = {'state', 'trans'}
    for node_ID, d in autjs['nodes'].iteritems():
        node_label = {k: d[k] for k in d if k not in omit}
        node_label['state'] = dict([(symtab[i].keys()[0],
                                     autjs['nodes'][node_ID]['state'][i])
                                    for i in range(len(symtab))])
        A.add_node(node_ID, node_label)
    for node_ID, d in autjs['nodes'].iteritems():
        for to_node in d['trans']:
            A.add_edge(node_ID, to_node)
    return A

def check_syntax(spec_str):
    """Check whether given string has correct gr1c specification syntax.

    Return True if syntax check passed, False on error.
    """
    _assert_gr1c()
    f = tempfile.TemporaryFile()
    f.write(spec_str)
    f.seek(0)

    p = subprocess.Popen([GR1C_BIN_PREFIX+"gr1c", "-s"],
                         stdin=f,
                         stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
    p.wait()

    logger.debug('gr1c returncode: ' + str(p.returncode) )
    logger.debug('gr1c stdout: ' + p.stdout.read() )

    if p.returncode == 0:
        return True
    else:
        logger.info(p.stdout.read() )
        return False

def check_realizable(spec):
    """Decide realizability of specification.

    Consult the documentation of L{synthesize} about parameters.

    @return: True if realizable, False if not, or an error occurs.
    """
    logger.info('checking realizability...')
    _assert_gr1c()
    init_option = select_options(spec)
    s = translate(spec, 'gr1c')
    f = tempfile.TemporaryFile()
    f.write(s)
    f.seek(0)
    logger.info('starting realizability check')
    p = subprocess.Popen([GR1C_BIN_PREFIX+"gr1c", "-n", init_option, "-r"],
                         stdin=f,
                         stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
    p.wait()

    logger.info('gr1c input:\n' + s +_hl)

    if p.returncode == 0:
        return True
    else:
        logger.info(p.stdout.read() )
        return False

def synthesize(spec):
    """Synthesize strategy realizing the given specification.

    @type spec: L{GRSpec}
    @param spec: specification.

    Consult the U{documentation of gr1c
    <https://tulip-control.github.io/gr1c/md_spc_format.html#initconditions>}
    for a detailed description.

    @return: strategy as C{networkx.DiGraph},
        or None if unrealizable or error occurs.
    """
    _assert_gr1c()
    init_option = select_options(spec)
    try:
        p = subprocess.Popen(
            [GR1C_BIN_PREFIX + "gr1c",
             "-n", init_option,
             "-t", "json"],
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE, stderr=subprocess.STDOUT
        )
    except OSError as e:
        if e.errno == os.errno.ENOENT:
            raise Exception('gr1c not found in path.')
        else:
            raise

    s = translate(spec, 'gr1c')
    logger.info('\n{hl}\n gr1c input:\n {s}\n{hl}'.format(s=s, hl=_hl))

    # to make debugging by manually running gr1c easier
    fname = 'spec.gr1c'
    try:
        if logger.getEffectiveLevel() < logging.DEBUG:
            with open(fname, 'w') as f:
                f.write(s)
            logger.debug('wrote input to file "{f}"'.format(f=fname))
    except:
        logger.error('failed to write auxiliary file: "{f}"'.format(f=fname))

    (stdoutdata, stderrdata) = p.communicate(s)

    msg = (
        ('{spaces} gr1c return code: {c}\n\n'
         '{spaces} gr1c stdout, stderr:\n {out}\n\n').format(
             c=p.returncode, out=stdoutdata, spaces=30 * ' '
        )
    )

    if p.returncode == 0:
        logger.debug(msg)
        strategy = load_aut_json(stdoutdata)
        return strategy
    else:
        print(msg)
        return None


def select_options(spec):
    """Return `gr1c` initial option based on `GRSpec` inits."""
    assert not spec.moore
    assert not spec.plus_one
    if spec.qinit == '\A \E':
        init_option = 'ALL_ENV_EXIST_SYS_INIT'
    elif spec.qinit == '\E \A':
        raise ValueError(
            '`qinit = "\E \A"` not supported by `gr1c`. '
            'Use `qinit = "\A \E"`.')
    elif spec.qinit == '\A \A':
        assert not spec.sys_init, spec.sys_init
        init_option = 'ONE_SIDE_INIT'
    elif spec.qinit == '\E \E':
        assert not spec.env_init, spec.env_init
        init_option = 'ONE_SIDE_INIT'
    else:
        raise ValueError(
            'unknown option `qinit = {qinit}`.'.format(
                qinit=spec.qinit))
    return init_option


def load_mealy(filename, fformat='tulipxml'):
    """Load C{gr1c} strategy from file.

    @param filename: file name
    @type filename: C{str}

    @param fformat: file format; can be one of "tulipxml" (default),
        "json". Not case sensitive.

    @type fformat: C{str}

    @return: loaded strategy as an annotated graph.
    @rtype: C{networkx.Digraph}
    """
    s = open(filename, 'r').read()
    if fformat.lower() == 'tulipxml':
        strategy = load_aut_xml(s)
    elif fformat.lower() == 'json':
        strategy = load_aut_json(s)
    else:
        ValueError('gr1c.load_mealy() : Unrecognized file format, "'
                   +str(fformat)+'"')

    logger.debug(
        'Loaded strategy with nodes: \n' + str(strategy.nodes()) +
        '\nand edges: \n' + str(strategy.edges())
    )
    return strategy

class GR1CSession:
    """Manage interactive session with gr1c.

    Given lists of environment and system variable names determine the
    order of values in state vectors for communication with the gr1c
    process.  Eventually there may be code to infer this directly from
    the spec file.

    **gr1c is assumed not to use GNU Readline.**

    Please compile it that way if you are using this class.
    (Otherwise, GNU Readline will echo commands and make interaction
    with gr1c more difficult.)

    The argument `prompt` is the string printed by gr1c to indicate it
    is ready for the next command.  The default value is a good guess.

    Unless otherwise indicated, command methods return True on
    success, False if error.
    """
    def __init__(self, spec_filename, sys_vars, env_vars=[], prompt=">>> "):
        self.spec_filename = spec_filename
        self.sys_vars = sys_vars[:]
        self.env_vars = env_vars[:]
        self.prompt = prompt
        if self.spec_filename is not None:
            self.p = subprocess.Popen([GR1C_BIN_PREFIX+"gr1c",
                                       "-i", self.spec_filename],
                                      stdin=subprocess.PIPE,
                                      stdout=subprocess.PIPE,
                                      stderr=subprocess.STDOUT)
        else:
            self.p = None

    def iswinning(self, state):
        """Return True if given state is in winning set, False otherwise.

        state should be a dictionary with keys of variable names
        (strings) and values of the value taken by that variable in
        this state, e.g., as in nodes of the Automaton class.
        """
        state_vector = range(len(state))
        for ind in range(len(self.env_vars)):
            state_vector[ind] = state[self.env_vars[ind]]
        for ind in range(len(self.sys_vars)):
            state_vector[ind+len(self.env_vars)] = state[self.sys_vars[ind]]
        self.p.stdin.write(
            "winning " +
            " ".join([str(i) for i in state_vector])+"\n"
        )
        if "True\n" in self.p.stdout.readline():
            return True
        else:
            return False

    def getindex(self, state, goal_mode):
        if goal_mode < 0 or goal_mode > self.numgoals()-1:
            raise ValueError("Invalid goal mode requested: "+str(goal_mode))
        state_vector = range(len(state))
        for ind in range(len(self.env_vars)):
            state_vector[ind] = state[self.env_vars[ind]]
        for ind in range(len(self.sys_vars)):
            state_vector[ind+len(self.env_vars)] = state[self.sys_vars[ind]]
        self.p.stdin.write("getindex "+" ".join(
            [str(i) for i in state_vector]) +" " +
            str(goal_mode) +"\n"
        )
        line = self.p.stdout.readline()
        if len(self.prompt) > 0:
                loc = line.find(self.prompt)
                if loc >= 0:
                    line = line[len(self.prompt):]
        return int(line[:-1])

    def env_next(self, state):
        """Return list of possible next environment moves, given current state.

        Format of given state is same as for iswinning method.
        """
        state_vector = range(len(state))
        for ind in range(len(self.env_vars)):
            state_vector[ind] = state[self.env_vars[ind]]
        for ind in range(len(self.sys_vars)):
            state_vector[ind+len(self.env_vars)] = state[self.sys_vars[ind]]
        self.p.stdin.write(
            "envnext " +" ".join([str(i) for i in state_vector]) +"\n"
        )
        env_moves = []
        line = self.p.stdout.readline()
        while "---\n" not in line:
            if len(self.prompt) > 0:
                loc = line.find(self.prompt)
                if loc >= 0:
                    line = line[len(self.prompt):]
            env_moves.append(dict([
                (k, int(s)) for (k,s) in
                zip(self.env_vars, line.split())
            ]))
            line = self.p.stdout.readline()
        return env_moves

    def sys_nextfeas(self, state, env_move, goal_mode):
        """Return list of next system moves consistent with some strategy.

        Format of given state and env_move is same as for iswinning
        method.
        """
        if goal_mode < 0 or goal_mode > self.numgoals()-1:
            raise ValueError("Invalid goal mode requested: "+str(goal_mode))
        state_vector = range(len(state))
        for ind in range(len(self.env_vars)):
            state_vector[ind] = state[self.env_vars[ind]]
        for ind in range(len(self.sys_vars)):
            state_vector[ind+len(self.env_vars)] = state[self.sys_vars[ind]]
        emove_vector = range(len(env_move))
        for ind in range(len(self.env_vars)):
            emove_vector[ind] = env_move[self.env_vars[ind]]
        self.p.stdin.write("sysnext "+" ".join(
            [str(i) for i in state_vector]
        )+" "+" ".join(
            [str(i) for i in emove_vector]
        )+" "+str(goal_mode)+"\n")
        sys_moves = []
        line = self.p.stdout.readline()
        while "---\n" not in line:
            if len(self.prompt) > 0:
                loc = line.find(self.prompt)
                if loc >= 0:
                    line = line[len(self.prompt):]
            sys_moves.append(dict([
                (k, int(s)) for (k,s)
                in zip(self.sys_vars, line.split())
            ]))
            line = self.p.stdout.readline()
        return sys_moves

    def sys_nexta(self, state, env_move):
        """Return list of possible next system moves, whether or not winning.

        Format of given state and env_move is same as for iswinning
        method.
        """
        state_vector = range(len(state))
        for ind in range(len(self.env_vars)):
            state_vector[ind] = state[self.env_vars[ind]]
        for ind in range(len(self.sys_vars)):
            state_vector[ind+len(self.env_vars)] = state[self.sys_vars[ind]]
        emove_vector = range(len(env_move))
        for ind in range(len(self.env_vars)):
            emove_vector[ind] = env_move[self.env_vars[ind]]
        self.p.stdin.write("sysnexta "+" ".join(
            [str(i) for i in state_vector]
        )+" "+" ".join(
            [str(i) for i in emove_vector])+"\n"
        )
        sys_moves = []
        line = self.p.stdout.readline()
        while "---\n" not in line:
            if len(self.prompt) > 0:
                loc = line.find(self.prompt)
                if loc >= 0:
                    line = line[len(self.prompt):]
            sys_moves.append(dict([
                (k, int(s)) for (k,s) in
                zip(self.sys_vars, line.split())
            ]))
            line = self.p.stdout.readline()
        return sys_moves

    def getvars(self):
        """Return string of environment and system variable names in order.

        Indices are indicated in parens.
        """
        self.p.stdin.write("var\n")
        line = self.p.stdout.readline()
        if len(self.prompt) > 0:
                loc = line.find(self.prompt)
                if loc >= 0:
                    line = line[len(self.prompt):]
        return line[:-1]

    def numgoals(self):
        self.p.stdin.write("numgoals\n")
        line = self.p.stdout.readline()
        if len(self.prompt) > 0:
                loc = line.find(self.prompt)
                if loc >= 0:
                    line = line[len(self.prompt):]
        return int(line[:-1])

    def reset(self, spec_filename=None):
        """Quit and start anew, reading spec from file with given name.

        If no filename given, then use previous one.
        """
        if self.p is not None:
            self.p.stdin.write("quit\n")
            returncode = self.p.wait()
            self.p = None
            if returncode != 0:
                self.spec_filename = None
                return False
        if spec_filename is not None:
            self.spec_filename = spec_filename
        if self.spec_filename is not None:
            self.p = subprocess.Popen(
                [GR1C_BIN_PREFIX+"gr1c",
                "-i", self.spec_filename],
                stdin=subprocess.PIPE,
                stdout=subprocess.PIPE,
                stderr=subprocess.STDOUT
            )
        else:
            self.p = None
        return True

    def close(self):
        """End session, and kill gr1c child process.
        """
        self.p.stdin.write("quit\n")
        returncode = self.p.wait()
        self.p = None
        if returncode != 0:
            return False
        else:
            return True



#!/usr/bin/env python
"""Direct GR(1) specification, with integer-valued variable to model location.

This example illustrates the use of TuLiP to synthesize a reactive
controller for a GR(1) specification.  We code the specification
directly in GR(1) form and then use TuLiP to synthesize a reactive
controller.

The system is modeled as a discrete transition system in which the
robot can be located anyplace on a 2x3 grid of cells:

    +----+----+----+
    | X3 | X4 | X5 |
    +----+----+----+
    | X0 | X1 | X2 |
    +----+----+----+

The robot is allowed to transition between any two adjacent cells;
diagonal motions are not allowed.  The robot should continuously
revisit the cell X5.

The environment consists of a single state called 'park' that
indicates that the robot should move to cell X0.

The system specification in its simplest form is given by

  []<>park -> []<>X5 && [](park -> <>X0)

We must convert this specification into GR(1) form:

  env_init && []env_safe && []<>env_prog_1 && ... && []<>env_prog_m ->
      sys_init && []sys_safe && []<>sys_prog_1 && ... && []<>sys_prog_n
"""
# 21 Jul 2013, Richard M. Murray (murray@cds.caltech.edu)

# Import the packages that we need
from tulip import spec
from tulip import synth


#
# Environment specification
#
# The environment can issue a park signal that the robot must respond
# to by moving to the lower left corner of the grid.  We assume that
# the park signal is turned off infinitely often.
#
env_vars = {'park'}
env_init = set()                # empty set
env_safe = set()                # empty set
env_prog = '!park'              # []<>(!park)

#
# System dynamics
#
# The system specification describes how the system is allowed to move
# and what the system is required to do in response to an environmental
# action.
#
sys_vars = {}
sys_vars['loc'] = (0, 5)

sys_init = {'loc=0'}
sys_safe = {
    'loc=0 -> X (loc=1 || loc=3)',
    'loc=1 -> X (loc=0 || loc=4 || loc=2)',
    'loc=2 -> X (loc=1 || loc=5)',
    'loc=3 -> X (loc=0 || loc=4)',
    'loc=4 -> X (loc=3 || loc=1 || loc=5)',
    'loc=5 -> X (loc=4 || loc=2)',
}
sys_prog = set()                # empty set

#
# System specification
#
# The system specification is that the robot should repeatedly revisit
# the upper right corner of the grid while at the same time responding
# to the park signal by visiting the lower left corner.  The LTL
# specification is given by
#
#     []<> X5 && [](park -> <>(loc=0))
#
# Since this specification is not in GR(1) form, we introduce an
# environment variable X0reach that is initialized to True and the
# specification [](park -> <>(loc=0)) becomes
#
#     [](X (X0reach) <-> (loc=0) || (X0reach && !park))
#

# Augment the system description to make it GR(1)
sys_vars['X0reach'] = 'boolean'
sys_init |= {'X0reach'}
sys_safe |= {'(X (X0reach) <-> (loc=0)) || (X0reach && !park)'}
sys_prog |= {'X0reach', 'loc=5'}

# Create a GR(1) specification
specs = spec.GRSpec(env_vars, sys_vars, env_init, sys_init,
                    env_safe, sys_safe, env_prog, sys_prog)
specs.qinit = '\E \A'  # Moore initial condition synthesized too

#
# Controller synthesis
#
# At this point we can synthesize the controller using one of the available
# methods.
#

ctrl = synth.synthesize('omega', specs)
assert ctrl is not None, 'unrealizable'


# Generate a graphical representation of the controller for viewing
if not ctrl.save('gr1_set.png'):
    print(ctrl)



# Copyright (c) 2013 by California Institute of Technology
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the California Institute of Technology nor
#    the names of its contributors may be used to endorse or promote
#    products derived from this software without specific prior
#    written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL CALTECH
# OR THE CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
# USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
# OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
"""
Convert Finite State Machines to State Chart XML (SCXML)
"""
def mealy2scxml(mealy):
    """Convert Mealy machine to SCXML.

    Using examples/transys/machine_examples:

    >>> from machine_examples import garage_counter
    >>> from tulip.transys.export import machine2scxml
    >>> m = garage_counter()
    >>> s = machine2scxml.mealy2scxml(m)
    >>> f = open('mealy.scxml', 'w')
    >>> f.write(s)
    >>> f.close()

    See Also
    ========
    transys.machines.mealy

    @param mealy: machine to export as SCXML
    @type mealy: MealyMachine

    @rtype: SCXML str
    """
    def indent(n):
        return '\n' +n*'\t'

    def transitions_str(from_state, mealy):
        s = ''
        trans = mealy.transitions.find([from_state] )
        n = 2

        for (from_state_, to_state, sublabel_dict) in trans:
            s += indent(n) +'<transition '
            n = n +1
            s += indent(n) +'event="input_present" '
            s += indent(n) +'cond="' +str(sublabel_dict) +'" '
            s += indent(n) +'target="' +str(to_state) +'"/>'
            n = n -1
            s += indent(n) +'</transition>'
        return s

    s = '<?xml version="1.0" encoding="UTF-8"?>\n'
    s += '<scxml xmlns="http://www.w3.org/2005/07/scxml" '
    s += ' version="1.0" '

    if len(mealy.states.initial) != 1:
        msg = 'Must have exactly 1 initial state.\n'
        msg += 'Got instead:\n\t' +str(mealy.states.initial() )
        raise Exception(msg)

    initial_state = mealy.states.initial()[0]
    s += 'initial="' +str(initial_state) +'">\n'

    for state in mealy.states():
        s += '\t<state id="' +str(state) +'">'
        s += transitions_str(state, mealy) +'\n'
        s += '\t</state>\n'

    s += '</scxml>'
    return s



# Copyright (c) 2015 by California Institute of Technology
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the California Institute of Technology nor
#    the names of its contributors may be used to endorse or promote
#    products derived from this software without specific prior
#    written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL CALTECH
# OR THE CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
# USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
# OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
"""Interface to C{gr1py}.

U{https://pypi.python.org/pypi/gr1py}
U{https://github.com/slivingston/gr1py}
"""
from __future__ import absolute_import
import logging
from tulip.spec import translate
from tulip.interfaces.gr1c import load_aut_json
from tulip.interfaces.gr1c import select_options
try:
    import gr1py
    import gr1py.cli
except ImportError:
    gr1py = None


logger = logging.getLogger(__name__)
_hl = 60 * '-'


def check_realizable(spec):
    """Decide realizability of specification.

    Consult the documentation of L{synthesize} about parameters.

    @return: True if realizable, False if not, or an error occurs.
    """
    init_option = select_options(spec)
    tsys, exprtab = _spec_to_gr1py(spec)
    return gr1py.solve.check_realizable(
        tsys, exprtab, init_flags=init_option)

def synthesize(spec):
    """Synthesize strategy realizing the given specification.

    cf. L{tulip.interfaces.gr1c.synthesize}
    """
    init_option = select_options(spec)
    tsys, exprtab = _spec_to_gr1py(spec)
    strategy = gr1py.solve.synthesize(
        tsys, exprtab, init_flags=init_option)
    if strategy is None:
        return None
    s = gr1py.output.dump_json(tsys.symtab, strategy)
    return load_aut_json(s)


def _spec_to_gr1py(spec):
    if gr1py is None:
        raise ValueError('Import of gr1py interface failed.\n'
                         'Please verify installation of "gr1py".')
    s = translate(spec, 'gr1c')
    logger.info('\n{hl}\n gr1py input:\n {s}\n{hl}'.format(s=s, hl=_hl))
    tsys, exprtab = gr1py.cli.loads(s)
    return tsys, exprtab



import logging
logging.basicConfig(level=logging.DEBUG)
logging.getLogger('tulip.ltl_parser_log').setLevel(logging.ERROR)
from nose.tools import raises
#from tulip.spec.parser import parse
from tulip import spec
from tulip.spec import translation as ts
from tulip.spec import form


def test_translate_ast_to_gr1c():
    x = '(loc = "s2") -> X((((env_alice = "left") && (env_bob = "bright"))))'
    s = spec.GRSpec(sys_vars={'loc': ['s0', 's2'],
                              'env_alice': ['left', 'right'],
                              'env_bob': ['bleft', 'bright']},
                    sys_safety=[x])
    s.str_to_int()
    sint = s._bool_int[x]
    print(repr(sint))
    rint = s.ast(sint)
    print(repr(rint))
    r = ts.translate_ast(rint, 'gr1c')
    print(repr(r))
    print(r.flatten())
    assert r.flatten() == ("( ( loc = 1 ) -> "
                           "( ( env_alice' = 0 ) & ( env_bob' = 1 ) ) )")


@raises(TypeError)
def check_translate_unrecognized_types(spc):
    ts.translate(spc, 'gr1c')

def test_translate_unrecognized_types():
    for spc in [form.LTL(), 'a -> b']:
        yield check_translate_unrecognized_types, spc



# Copyright (c) 2013-2014 by California Institute of Technology
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the California Institute of Technology nor
#    the names of its contributors may be used to endorse or promote
#    products derived from this software without specific prior
#    written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL CALTECH
# OR THE CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
# USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
# OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
"""
Convenience functions for plotting

WARNING: The public functions dimension, newax, dom2vec, quiver will
eventually be removed.  Their use in new applications is discouraged.

They come from https://github.com/johnyf/pyvectorized
"""
from __future__ import division

import logging
logger = logging.getLogger(__name__)

from warnings import warn
from itertools import izip_longest

import numpy as np
try:
    from matplotlib import pyplot as plt
    from mpl_toolkits.mplot3d import axes3d
except Exception as e:
    plt = None
    logger.error(e)

# from mayavi import mlab

def dimension(ndarray):
    """dimension of ndarray  (DEPRECATED)

    - ndim == 1:
        dimension = 1
    - ndim == 2:
        dimension = shape[0]
    """
    if ndarray.ndim < 2:
        return ndarray.ndim
    return ndarray.shape[0]

def newax(subplots=(1, 1), fig=None,
          mode='list', dim=2):
    """Create (possibly multiple) new axes handles.  (DEPRECATED)

    @param fig: attach axes to this figure
    @type fig: figure object,
        should be consistent with C{dim}

    @param subplots: number or layout of subplots
    @type subplots: int or
        2-tuple of subplot layout

    @param mode: return the axes shaped as a
        vector or as a matrix.
        This is a convenience for later iterations
        over the axes.
    @type mode: 'matrix' | ['list']

    @param dim: plot dimension:

            - if dim == 2, then use matplotlib
            - if dim == 3, then use mayavi

        So the figure type depends on dim.

    @return: C{(ax, fig)} where:
        - C{ax}: axes created
        - C{fig}: parent of ax
    @rtype: list or list of lists,
        depending on C{mode} above
    """
    assert_pyplot()
    # layout or number of axes ?
    try:
        subplot_layout = tuple(subplots)
    except:
        subplot_layout = (1, subplots)

    # reasonable layout ?
    if len(subplot_layout) != 2:
        raise Exception('newax:'
                        'subplot layout should be 2-tuple or int.')

    # which figure ?
    if fig is None:
        fig = plt.figure()

    # create subplot(s)
    (nv, nh) = subplot_layout
    n = np.prod(subplot_layout)

    try:
        dim = tuple(dim)
    except:
        # all same dim
        dim = [dim] * n

    # matplotlib (2D) or mayavi (3D) ?
    ax = []
    for (i, curdim) in enumerate(dim):
        if curdim == 2:
            curax = fig.add_subplot(nv, nh, i + 1)
            ax.append(curax)
        else:
            curax = fig.add_subplot(nv, nh, i + 1, projection='3d')
            ax.append(curax)

        if curdim > 3:
            warn('ndim > 3, but plot limited to 3.')

    if mode is 'matrix':
        ax = list(_grouper(nh, ax))

    # single axes ?
    if subplot_layout == (1, 1):
        ax = ax[0]

    return (ax, fig)

def dom2vec(domain, resolution):
    """Matrix of column vectors for meshgrid points.  (DEPRECATED)

    Returns a matrix of column vectors for the meshgrid
    point coordinates over a parallelepiped domain
    with the given resolution.

    Example
    =======
    >>> domain = [0, 1, 0,2]
    >>> resolution = [4, 5]
    >>> q = domain2vec(domain, resolution)

    @param domain: extremal values of parallelepiped
    @type domain: [xmin, xmax, ymin, ymax, ...]

    @param resolution: # points /dimension
    @type resolution: [nx, ny, ...]

    @return: q = matrix of column vectors (meshgrid point coordinates)
    @rtype: [#dim x #points]

    See also vec2meshgrid, domain2meshgrid, meshgrid2vec.
    """
    domain = _grouper(2, domain)
    lambda_linspace = lambda (dom, res): np.linspace(dom[0], dom[1], res)
    axis_grids = map(lambda_linspace, zip(domain, resolution))
    pnt_coor = np.meshgrid(*axis_grids)
    q = np.vstack(map(np.ravel, pnt_coor))

    return q

def quiver(x, v, ax=None, **kwargs):
    """Multi-dimensional quiver.  (DEPRECATED)

    Plot v columns at points in columns of x
    in axes ax with plot formatting options in kwargs.

    >>> import numpy as np
    >>> import matplotlib as mpl
    >>> from pyvectorized import quiver, dom2vec
    >>> x = dom2vec([0, 10, 0, 11], [20, 20])
    >>> v = np.vstack(np.sin(x[1, :] ), np.cos(x[2, :] ) )
    >>> quiver(mpl.gca(), x, v)

    see also
        matplotlib.quiver, mayavi.quiver3

    @param x: points where vectors are based
        each column is a coordinate tuple
    @type x: 2d lil | numpy.ndarray

    @param v: vectors which to base at points x
    @type v: 2d lil | numpy.ndarray

    @param ax: axes handle, e.g., ax = gca())

    @param x: matrix of points where vectors are plotted
    @type x: [#dim x #points]

    @param v: matrix of column vectors to plot at points x
    @type v: [#dim x #points]

    @param kwargs: plot formatting

    @return: handle to plotted object(s)
    """
    assert_pyplot()
    # multiple axes ?
    try:
        fields = [quiver(x, v, i, **kwargs) for i in ax]
        return fields
    except:
        pass

    if not ax:
        ax = plt.gca()

    dim = dimension(x)

    if dim < 2:
        raise Exception('ndim < 2')
    elif dim < 3:
        h = ax.quiver(x[0, :], x[1, :],
                      v[0, :], v[1, :], **kwargs)
    else:
        raise NotImplementedError

        from mayavi.mlab import quiver3d

        if ax:
            print('axes arg ignored, mayavi used')

        h = quiver3d(x[0, :], x[1, :], x[2, :],
                     v[0, :], v[1, :], v[2, :], **kwargs)

    if dim > 3:
        warn('quiver:ndim #dimensions > 3,' +
             'plotting only 3D component.')

    return h

def _grouper(n, iterable, fillvalue=None):
    """grouper(3, 'ABCDEFG', 'x') --> ABC DEF Gxx
    """
    args = [iter(iterable)] * n
    return izip_longest(fillvalue=fillvalue, *args)


def assert_pyplot():
    if plt is None:
        raise ImportError('Failed to import `matplotlib.pyplot`')



#!/usr/bin/env python
"""Demonstration of FMU export from a controller synthesized using TuLiP

This is a modified copy from the TuLiP sources of
examples/robot_planning/continuous.py that exports an FMU.  The key
changes are

* caching of the abstraction and controller in AbstractPwa.p and FSM.p, and
* calling exportFMU() and `make test_controller` (near end of this file).
"""
import os.path
import os
import pickle
import sys
import numpy as np

from tulip import spec, synth, hybrid
from polytope import box2poly
from tulip.abstract import prop2part, discretize

from exportFMU import exportFMU


BUILDDIR="build/"

if os.path.isfile(BUILDDIR+'AbstractPwa.p') and os.path.isfile(BUILDDIR+'FSM.p'):
    pwa = pickle.load(open(BUILDDIR+"AbstractPwa.p", "rb"))
    ctrl = pickle.load(open(BUILDDIR+"FSM.p", "rb"))
else:
    # the code in robot_planning/continuous.py
    # Problem parameters
    input_bound = 1.0
    uncertainty = 0.01

    # Continuous state space
    cont_state_space = box2poly([[0., 3.], [0., 2.]])

    # Continuous dynamics
    A = np.array([[1.0, 0.], [0., 1.0]])
    B = np.array([[0.1, 0.], [0., 0.1]])
    E = np.array([[1., 0.], [0., 1.]])

    # Available control, possible disturbances
    U = input_bound * np.array([[-1., 1.], [-1., 1.]])
    W = uncertainty * np.array([[-1., 1.], [-1., 1.]])

    # Convert to polyhedral representation
    U = box2poly(U)
    W = box2poly(W)

    # Construct the LTI system describing the dynamics
    sys_dyn = hybrid.LtiSysDyn(A, B, E, None, U, W, cont_state_space)

    # Define atomic propositions for relevant regions of state space
    cont_props = {}
    cont_props['home'] = box2poly([[0., 1.], [0., 1.]])
    cont_props['lot'] = box2poly([[2., 3.], [1., 2.]])

    # Compute proposition preserving partition of the continuous state space
    cont_partition = prop2part(cont_state_space, cont_props)

    pwa = discretize(
        cont_partition, sys_dyn, closed_loop=True,
        N=8, min_cell_volume=0.1, plotit=False
    )

    """Specifications"""
    # Environment variables and assumptions
    env_vars = {'park'}
    env_init = set()                # empty set
    env_prog = '!park'
    env_safe = set()                # empty set

    # System variables and requirements
    sys_vars = {'X0reach'}
    sys_init = {'X0reach'}
    sys_prog = {'home'}               # []<>home
    sys_safe = {'(X(X0reach) <-> lot) || (X0reach && !park)'}
    sys_prog |= {'X0reach'}

    # Create the specification
    specs = spec.GRSpec(env_vars, sys_vars, env_init, sys_init,
                        env_safe, sys_safe, env_prog, sys_prog)

    """Synthesize"""
    ctrl = synth.synthesize('gr1c', specs,
                            sys=pwa.ts, ignore_sys_init=True)
    # end of the code in robot_planning/continuous.py

    # store the result for future use
    if len(BUILDDIR) > 0 and not os.path.exists(BUILDDIR):
        os.mkdir(BUILDDIR)
    pickle.dump(ctrl, open(BUILDDIR+'FSM.p', 'wb'))
    pickle.dump(pwa, open(BUILDDIR+'AbstractPwa.p', 'wb'))

x0 = np.array([1.5, 1.5])
d0 = 18

exportFMU(ctrl, pwa, x0, d0)
os.system("make test_controller")



"""Collect and print environment information."""
from __future__ import absolute_import
import subprocess

try:
    import cvxopt
except ImportError:
    cvxopt = None
try:
    from cvxopt import glpk as cvxopt_glpk
except ImportError:
    cvxopt_glpk = None
try:
    import scipy
except ImportError:
    scipy = None
try:
    import numpy
except ImportError:
    numpy = None
try:
    import polytope
except ImportError:
    polytope = None


from tulip.interfaces import gr1c as gr1c
from tulip.interfaces import gr1py as gr1py_int
from tulip.interfaces import omega as omega_int


def print_env():
    """Print to `stdout` relevant information about environment.

    Looks for solvers like `gr1c` and `glpk`, bindings with `cvxopt`,
    and reports this and other relevant information.
    """
    c = list()
    s = '---- ENVIRONMENT INFORMATION RELEVANT TO `tulip` ----\n'
    c.append(s)
    if gr1c.check_gr1c():
        s = 'Found `gr1c` in `$PATH`.\n'
    else:
        s = 'Did not find `gr1c` in `$PATH`.\n'
    c.append(s)
    s = _format_python_package_message(
        'gr1py', gr1py_int.gr1py, 'https://pypi.python.org/pypi/gr1py')
    c.append(s)
    s = _format_python_package_message(
        'omega', omega_int.omega, 'https://pypi.python.org/pypi/omega')
    c.append(s)
    s = _format_python_package_message(
        'dd.cudd', omega_int.cudd, 'https://pypi.python.org/pypi/dd')
    c.append(s)
    s = _format_python_package_message(
        'numpy', numpy, 'https://pypi.python.org/pypi/numpy')
    c.append(s)
    s = _format_python_package_message(
        'scipy', scipy, 'https://pypi.python.org/pypi/scipy')
    c.append(s)
    s = _format_python_package_message(
        'cvxopt', cvxopt, 'https://pypi.python.org/pypi/cvxopt')
    c.append(s)
    if _check_glpsol():
        s = 'Found GLPK solver `glpsol` in `$PATH`.\n'
    else:
        s = 'Did not find `glpsol` in `$PATH`.\n'
    c.append(s)
    if cvxopt_glpk is None:
        s = (
            'Could not import module `cvxopt.glpk`.\n'
            'Can be installed by compiling and linking `cvxopt` to GLPK.\n')
    else:
        s = 'Found module `cvxopt.glpk` as:\n    {cvxopt_glpk}\n.'.format(
            cvxopt_glpk=cvxopt_glpk)
    c.append(s)
    s = _format_python_package_message(
        'polytope', polytope, 'https://pypi.python.org/pypi/polytope')
    c.append(s)
    s = [
        'For details about what each package and solver does, '
        'see the documentation. In summary:',
        '',
        '---- DISCRETE SYNTHESIS ----',
        '`omega` is a Python solver for discrete synthesis,',
        '`gr1c` is a synthesizer written in C that uses the library CUDD.',
        'If `dd.cudd` has been built, then `omega` can use the C library '
        'CUDD for more efficient operation.',
        '',
        '---- CONTINUOUS SYNTHESIS ----',
        '`polytope` is a Python package that uses `scipy` and `numpy`.',
        'If `cvxopt` is installed, GLPK available, and '
        'cvxopt.glpk` compiled and linked successfully, ',
        'then `polytope` will use `cvxopt.glpk`, which is notably faster.',
        '',
        50 * '=']
    c.extend(s)
    s = '\n'.join(c)
    print(s)


def _format_python_package_message(name, package, url):
    """Return a `str` reporting information about package."""
    if package is None:
        s = (
            'Could not import Python package `{name}`.\n'
            'Can be installed from:\n    `{url}`\n').format(
                name=name, url=url)
    else:
        s = (
            'Found Python package `{name}` as:\n'
            '    {gr1py}\n').format(
                name=name, gr1py=package)
    return s


def _check_glpsol():
    """Return `True` if `glpsol` (of GLPK) in `$PATH`."""
    try:
        v = subprocess.check_output(['glpsol', '--version'])
    except OSError:
        return False
    return True



#!/usr/bin/env python
"""
Driver script for testing TuLiP.  Try calling it with "-h" flag.
"""
from __future__ import print_function
import imp
import sys
import os
from os import walk
import argparse

import nose
from nose.plugins.base import Plugin


class ShowFunctions(Plugin):
    name = 'showfunctions'
    def describeTest(self, test):
        return str(test)


class ArgParser(argparse.ArgumentParser):
    def error(self, message):
        sys.stderr.write('error: %s\n' % message)
        self.print_help()
        sys.exit(1)

testfamilies_help = (
"""
name of a family of tests. If the `-f` or `--testfiles`
switch is not used, then at most one of the following
can be requested (not case sensitive):

* "base": (default) tests that should pass on a
  basic TuLiP installation with required dependencies.

* "hybrid": tests that should pass given dependencies
  implied by the "hybrid" extras: polytope.

* "full": all tests. Every optional dependency or
  external tool is potentially used.

If the `-f` or `--testfiles` switch is used, then
TEST [TEST ...] are interpreted as described below."""
)

testfiles_help = (
"""
space-separated list of test file names,
where the suffix "_test.py" is added to
each given name.  E.g.,

    run_tests.py gr1cint

causes the gr1cint_test.py file to be used
and no others.

If no arguments are given,
then default is to run all tests.

If TESTFILES... each have a prefix of "-",
then all tests *except* those listed will be run.
Use the pseudoargument -- in this case. E.g.,

    run_tests.py --fast -- -abstract -synth

Besides what is below, OPTIONS... are passed on to nose."""
)


def main():
    parser = ArgParser(formatter_class=argparse.RawTextHelpFormatter)

    parser.add_argument('testfamily', metavar='TEST', nargs='*', default=['base'],
                        help=testfamilies_help)
    parser.add_argument('-f,--testfiles', dest='testfiles', action='store_true',
                        help=testfiles_help)
    parser.add_argument('--fast', action='store_true',
                        help='exclude tests that are marked as slow')
    parser.add_argument('--cover', action='store_true',
                        help='generate a coverage report')
    parser.add_argument('--outofsource', action='store_true',
                        help='import tulip from outside the current directory')
    parser.add_argument(
        '-w', '--where', dest='dir', default='tests',
        help='search for tests in directory WHERE\n'
             '(this is exactly the "-w" or "--where" option of nose)')

    args, unknown_args = parser.parse_known_args()

    if (not args.testfiles) and len(args.testfamily) > 1:
        print('At most one family of tests can be requested.')
        sys.exit(1)

    if args.testfiles:
        basenames = args.testfamily
    else:
        basenames = None
        args.testfamily = args.testfamily[0]
    skip_slow = args.fast
    measure_coverage = args.cover
    require_nonlocaldir_tulip = args.outofsource
    tests_dir = args.dir

    if require_nonlocaldir_tulip:
        # Scrub local directory from search path for modules
        try:
            while True:
                sys.path.remove('')
        except ValueError:
            pass
        try:
            while True:
                sys.path.remove(os.path.abspath(os.curdir))
        except ValueError:
            pass
    try:
        modtuple = imp.find_module("tulip", sys.path)
        imp.load_module("tulip", *modtuple)
    except ImportError:
        if require_nonlocaldir_tulip:
            raise ImportError('tulip package not found, '
                              'besides in the local directory')
        else:
            raise()

    argv = ["nosetests"]
    if skip_slow:
        argv.append("--attr=!slow")

    if measure_coverage:
        argv.extend(["--with-coverage", "--cover-html", "--cover-package=tulip"])

    available = []
    for dirpath, dirnames, filenames in walk(tests_dir):
        available.extend(filenames)
        break

    testfiles = []
    excludefiles = []

    if args.testfiles:
        for basename in basenames:
            if basename[0] == '-':
                matchstart = lambda f, bname: f.startswith(bname[1:])
            else:
                matchstart = lambda f, bname: f.startswith(bname)
            match = [f for f in available
                     if matchstart(f, basename) and f.endswith('.py')]

            if len(match) > 1:
                raise Exception('ambiguous base name: %s, matches: %s' %
                                (basename, match))
            elif len(match) == 1 and match[0].endswith('_test.py'):
                if basename[0] == '-':
                    excludefiles.append(match[0])
                else:
                    testfiles.append(match[0])
                continue

            if os.path.exists(os.path.join(tests_dir, basename + '_test.py')):
                testfiles.append(basename + '_test.py')
            elif basename[0] == '-':
                if os.path.exists(os.path.join(tests_dir, basename[1:] + "_test.py")):
                    excludefiles.append(basename[1:] + "_test.py")
                else:
                    argv.append(basename)
            else:
                argv.append(basename)

        if testfiles and excludefiles:
            print("You can specify files to exclude or include, but not both.")
            print("Try calling it with \"-h\" flag.")
            exit(1)

        if excludefiles:
            argv.append("--exclude=" + "|".join(excludefiles))

    else:
        base = [
            'dumpsmach_test',
            'form_test',
            'gr1_test',
            'omega_interface_test',
            'spec_test',
            'synth_test',
            'transform_test',
            'translation_test',
            'transys_automata_test',
            'transys_labeled_graphs_test',
            'transys_machines_test',
            'transys_mathset_test',
            'transys_ts_test',
            'gridworld_test']
        hybrid = [
            'abstract_test',
            'hybrid_test',
            'prop2part_test']
        if args.testfamily.lower() == 'base':
            testfiles = base
        elif args.testfamily.lower() == 'hybrid':
            testfiles = base + hybrid
        elif args.testfamily.lower() == 'full':
            pass
        else:
            print('Unrecognized test family: "'+args.testfamily+'"')
            sys.exit(1)

    argv.extend(testfiles)
    argv += ["--where=" + tests_dir]
    argv.extend(unknown_args)

    print('calling nose')
    argv.extend(["--verbosity=3",
                 "--exe",
                 "--with-showfunctions"])
    nose.main(argv=argv,
              addplugins=[ShowFunctions()])


if __name__ == "__main__":
    main()



# Copyright (c) 2014 by California Institute of Technology
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the California Institute of Technology nor
#    the names of its contributors may be used to endorse or promote
#    products derived from this software without specific prior
#    written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL CALTECH
# OR THE CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
# USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
# OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.

"""
Contains XML import and export functions so that we can export (and not
recompute) Tulip data structures.
"""

import numpy
import scipy.sparse
import xml.etree.ElementTree as ET

import polytope
from tulip import transys
from tulip import hybrid
from tulip import abstract


# Global names used in tags
N_PROPS = 'props'
N_PROP = 'prop'
N_LOWERBOUND = 'lb'
N_UPPERBOUND = 'ub'
N_FULLDIM = 'fulldim'
N_MINREP = 'minrep'
N_VOLUME = 'volume'
N_DIMENSION = 'dim'
N_BBOX = 'bbox'
N_CHEBR = 'chebR'
N_CHEBXC = 'chebXc'
N_POLYLIST = 'list_poly'
N_REGIONLIST = 'list_reg'
N_KEY = 'key'
N_VALUE = 'value'
N_KEYVALUEPAIR = 'pair'
N_DOMAIN = 'domain'
N_PROPREG = 'proplist'
N_ADJ = 'adj'
N_ITEM = 'item'


# data types used in xml file (attributes)
T_SET = 'set'
T_STRING = 'str'
T_INT = 'int'
T_FLOAT = 'float'
T_BOOL = 'bool'
T_POLYTOPE = 'polytope'
T_MATRIX = 'ndarray'
T_REGION = 'region'
T_DICT = 'dict'
T_PPP = 'PPP'
T_ADJ = 'adjmatrix'
T_TUPLE = 'tuple'
T_LIST = 'list'
T_LTISYS = 'LtiSysDyn'
T_PWASYS = 'PwaSysDyn'
T_HYBRIDSYS = 'HybridSysDyn'
T_FTS = 'FiniteTransitionSystem'
T_OFTS = 'OpenFiniteTransitionSystem'
T_HYBRIDSYS = 'SwitchedSysDyn'


def _make_pretty(tree, indent=1):
	"""
	Modifies the tail attributes of nodes in an XML tree so that the resulting
	printed XML file looks like an indented tree (and not one long line).

	@type tree: xml.etree.ElementTree.Element
	@type indent: int

	@return: None, just modifies tree by reference
	@rtype: None
	"""

	tab_string = '\n' + '\t'*indent

	# If a tree has children, put tabs in front of the first child
	if tree.getchildren():
		if tree.text is not None:
			tree.text = tab_string + tree.text
		else:
			tree.text = tab_string

	# Number of children in the tree
	N = len(tree.getchildren())

	# Recursively run function on children
	for index, child in enumerate(tree):
		_make_pretty(child, indent=indent+1)

		# Print new line and tabs at the tail of each child, except the last
		# child. Print new line and one fewer tab at the tail of the last child.
		if index < N - 1:
			child.tail = tab_string
		else:
			child.tail = '\n' + '\t'*(indent-1)



def exportXML(data, filename, tag=None):
	"""
	Exports a Tulip data structure into an XML file.

	@param data: The Tulip data structure to export into an xml lfile
	@type data: L{Polytope}, L{Region}, L{PropPreservingPartition},
		L{SwitchedSysDyn}, L{PwaSysDyn}, L{LtiSysDyn}
	@param filename: The name of the XML file to export to.
	@type filename: string
	@param tag: (Optional) What we want the first tag of the XML file to read.
	@type tag: string

	@return: No return, just writes text.
	@rtype: None
	"""

	if tag is None:
		tag = "object"

	tree = _export_xml(data, None, tag)
	_make_pretty(tree)
	pretty_string = ET.tostring(tree)

	xmlfile = open(filename, 'w')
	xmlfile.write(pretty_string)
	xmlfile.close()



def importXML(filename):
	"""
	Takes a Tulip XML file and returns a Tulip data structure.

	@param filename: XML file containing exported data to import
	@type filename: string

	@return: the data structure exported into the file.
	@rtype: L{Polytope}, L{Region}, L{PropPreservingPartition},
		L{SwitchedSysDyn}, L{PwaSysDyn}, L{LtiSysDyn}
	"""

	# Load the file into an xml tree
	xmltree = ET.parse(filename)

	# send the root to the general tree parser
	return _import_xml(xmltree.getroot())



def _import_xml(node):
	"""
	Returns the Tulip data structure contained in a parsed XML file.

	@type tree: xml.etree.ElementTree.Element

	@rtype: L{Polytope}, L{Region}, L{PropPreservingPartition}, L{HybridSysDyn},
		L{PwaSysDyn}, L{LtiSysDyn}, L{FiniteTransitionSystem},
		L{OpenFiniteTransitionSystem}
	"""

	# Get the type of data this is
	nodetype = node.attrib['type']


	# Call the right import function

	# Python types
	if nodetype == T_STRING:
		return node.text
	elif nodetype == T_INT:
		return int(node.text)
	elif nodetype == T_FLOAT:
		return float(node.text)
	elif nodetype == T_BOOL:
		return bool(node.text)
	elif nodetype == T_MATRIX:
		return eval('numpy.array(' + node.text + ')')
	elif nodetype == T_TUPLE:
		return _import_list(node, type_str=T_TUPLE)
	elif nodetype == T_DICT:
		return _import_dictionary(node)
	elif nodetype == T_LIST:
		return _import_list(node)
	elif nodetype == T_ADJ:
		return _import_adj(node)
	elif nodetype == T_SET:
		return _import_list(node, type_STR=T_SET)

	# Tulip data structures
	elif nodetype == T_POLYTOPE:
		return _import_polytope(node)
	elif nodetype == T_REGION:
		return _import_region(node)
	elif nodetype == T_PPP:
		return _import_ppp(node)
	elif nodetype == T_LTISYS:
		return _import_ltisys(node)
	elif nodetype == T_PWASYS:
		return _import_pwasys(node)
	elif nodetype == T_HYBRIDSYS:
		return _import_hybridsys(node)
	else:
		raise TypeError('Type ' + nodetype + ' is not supported.')


def _import_adj(node):

	# Get number of rows and columns
	N = _import_xml(node.findall('num_states')[0])

	# Make matrix
	sparse_matrix = scipy.sparse.lil_matrix((N,N))

	# Get entries and fill them in with ones
	entries = _import_xml(node.findall('index_list')[0])
	for entry in entries:
		sparse_matrix[entry[0],entry[1]] = 1

	return sparse_matrix



def _import_ppp(node):

	# Domain
	domain_node = node.findall('domain')
	if domain_node:
		domain = _import_xml(domain_node[0])
	else:
		domain = None

	# Regions
	regions_node = node.findall('list_reg')
	if regions_node:
		list_regions = _import_xml(regions_node[0])
	else:
		list_regions = []

	# adj
	adj_node = node.findall('adj')
	if adj_node:
		adjmatrix = _import_xml(adj_node[0])
	else:
		adjmatrix = None

	# prop_regions
	prop_regions_node = node.findall('proplist')
	if prop_regions_node:
		prop_regions = _import_xml(prop_regions_node[0])
	else:
		prop_regions = None

	return abstract.prop2partition.PropPreservingPartition(domain=domain,
		regions=list_regions, adj=adjmatrix, prop_regions=prop_regions)




def _import_list(node, type_str=T_LIST):
	all_stuff = []
	for child in node:
		all_stuff.append(_import_xml(child))

	if type_str==T_SET:
		all_stuff = set(all_stuff)
	elif type_str==T_TUPLE:
		all_stuff = tuple(all_stuff)

	return all_stuff



def _import_dictionary(node):
	dictionary = {}
	for keyvaluepair in node:
		key = _import_xml(keyvaluepair.findall(N_KEY)[0])
		value = _import_xml(keyvaluepair.findall(N_VALUE)[0])
		dictionary[key] = value
	return dictionary


def _import_region(node):

	# Get the polytope list and import the polytopes
	polytope_list = node.findall(N_POLYLIST)[0]

	# Import the polytopes
	list_poly = _import_xml(polytope_list)

	return polytope.Region(list_poly=list_poly)


def _import_polytope(node):

	# Get the A matrix
	A = _import_xml(node.findall('A')[0])

	# Get the b matrix
	b = _import_xml(node.findall('b')[0])

	return polytope.Polytope(A=A, b=b)



def _import_ltisys(node):
	A = _import_xml(node.findall('A')[0])
	B = _import_xml(node.findall('B')[0])
	E = _import_xml(node.findall('E')[0])
	K = _import_xml(node.findall('K')[0])
	Uset = node.findall('Uset')
	Wset = node.findall('Wset')
	domain = node.findall('domain')

	if not Uset:
		Uset = None
	else:
		Uset = _import_xml(Uset[0])
	if not Wset:
		Wset = None
	else:
		Wset = _import_xml(Wset[0])
	if not domain:
		domain = None
	else:
		domain = _import_xml(domain[0])

	return hybrid.LtiSysDyn(A=A, B=B, E=E, K=K, Uset=Uset, Wset=Wset,
		domain=domain)



def _import_pwasys(node):

	domain = node.findall('domain')
	if domain:
		domain = _import_xml(domain[0])
	else:
		domain = None

	# Get list of ltisys
	ltilist = node.findall('ltilist')[0]
	list_subsys = _import_xml(ltilist)

	return hybrid.PwaSysDyn(list_subsys=list_subsys, domain=domain)



def _import_hybridsys(node):

	# Get parts, import the non-optional parts
	disc_domain_size = _import_xml(node.findall('disc_domain_size')[0])
	sys_labels = node.findall('sys_labels')
	env_labels = node.findall('env_labels')
	cts_ss = _import_xml(node.findall('cts_ss')[0])
	dynamics = _import_xml(node.findall('dynamics')[0])

	if sys_labels:
		sys_labels = _import_xml(sys_labels[0])
	else:
		sys_labels = None
	if env_labels:
		env_labels = _import_xml(env_labels[0])
	else:
		env_lables = None

	return hybrid.SwitchedSysDyn(disc_domain_size=disc_domain_size,
		dynamics=dynamics, cts_ss=cts_ss, env_labels=env_labels,
		disc_sys_labels=sys_labels)



def _export_xml(data, parent=None, tag=None, tag_list=[]):
	"""Exports Tulip data structures to XML structures for later import. This
	function is called both internal

	@param data: the data structure to be exported into an XML tree.
	@type data: numpy.ndarray or L{Polytope} or L{Region} or
	    L{FiniteTransitionSystem} or L{PropPreservingPartition} or
		L{AbstractSysDyn} or dict
	@param parent:
	@type parent: None or xml.etree.ElementTree.Element or
		xml.etree.ElementTree.SubElement
	@type tag: None or string

	@return: None (if parent is None), or an xml tree
	@rtype: None or xml.etree.ElementTree.Element or
		xml.etree.ElementTree.SubElement
	"""

	# Tulip types (parent might not exist)
	if isinstance(data, polytope.Polytope):
		if parent is None:
			return _export_polytope(data, parent, tag)
		else:
			_export_polytope(data, parent, tag)
	elif isinstance(data, polytope.Region):
		if parent is None:
			return _export_region(data, parent, tag)
		else:
			_export_region(data, parent, tag)

	elif isinstance(data, abstract.prop2partition.PropPreservingPartition):
		if parent is None:
			return _export_ppp(data, parent, tag)
		else:
			_export_ppp(data, parent, tag)

	elif isinstance(data, hybrid.LtiSysDyn):
		if parent is None:
			return _export_ltisys(data, parent, tag)
		else:
			_export_ltisys(data, parent, tag)

	elif isinstance(data, hybrid.PwaSysDyn):
		if parent is None:
			return _export_pwasys(data, parent, tag)
		else:
			_export_pwasys(data, parent, tag)

	elif isinstance(data, hybrid.SwitchedSysDyn):
		if parent is None:
			return _export_hybridsys(data, parent, tag)
		else:
			_export_hybridsys(data, parent, tag)

	elif isinstance(data, transys.transys.FTS):
		if parent is None:
			return _export_fts(data, parent, tag, type_str=T_FTS)
		else:
			_export_fts(data, parent, tag, type_str=T_FTS)

	elif isinstance(data, transys.transys.OpenFTS):
		if parent is None:
			return _export_fts(data, parent, tag, type_str=T_OFTS)
		else:
			_export_fts(data, parent, tag, type_str=T_OFTS)


	# parent will always not be none
	elif (isinstance(data, int) or isinstance(data, numpy.int32)):
		if tag is None:
			tag = "integer"
		new_node = ET.SubElement(parent, tag, type=T_INT)
		new_node.text = str(data)

	elif isinstance(data, str):
		if tag is None:
			tag = "string"
		new_node = ET.SubElement(parent, tag, type=T_STRING)
		new_node.text = data

	elif (isinstance(data, bool) or isinstance(data, numpy.bool_)):
		if tag is None:
			tag = "bool"
		new_node = ET.SubElement(parent, tag, type=T_BOOL)
		new_node.text = str(data)

	elif isinstance(data, float):
		if tag is None:
			tag = "float"
		new_node = ET.SubElement(parent, tag, type=T_FLOAT)
		new_node.text = str(data)

	elif isinstance(data, dict):
		_export_dict(data, parent, tag)

	elif isinstance(data, numpy.ndarray):
		if tag is None:
			tag = "numpyarray"
		new_node = ET.SubElement(parent, tag, type=T_MATRIX)
		new_node.text = str(data.tolist())

	elif isinstance(data, tuple):
		#_export_tuple(data, parent, tag)
		_export_list(data, parent, tag, type_str=T_TUPLE, tag_list=tag_list)

	elif isinstance(data, list):
		_export_list(data, parent, tag, type_str=T_LIST, tag_list=tag_list)

	elif isinstance(data, scipy.sparse.lil.lil_matrix):
		_export_adj(data, parent, tag)

	elif isinstance(data, set):
		#_export_set(data, parent, tag)
		_export_list(data, parent, tag, type_str=T_SET, tag_list=tag_list)

	# Type not found
	else:
		raise TypeError('Type ' + str(type(data)) + ' is not supported.')



def _export_fts(fts, parent, tag, type_str=T_OFTS):
	if tag is None:
		tag = "TransitionSystem"
	if parent is None:
		tree = ET.Element(tag, type=type_str)
	else:
		tree = ET.SubElement(parent, tag, type=type_str)

	# List of atomic propositions
	_export_list(fts.aps, tree, 'APs', type_str=T_SET)

	# List of states with labels
	N = len(fts.states.find())
	tag_list0 = [ 'state' for i in range(N) ]
	states_list_node = ET.SubElement(tree, 'states')
	for state in fts.states.find():
		state_node = ET.SubElement(states_list_node, 'state')
		name = state[0]
		ap_list = state[1]['ap']
		_export_xml(state[0], state_node, 'name')
		_export_xml(ap_list, state_node, 'aps')
#	_export_list(fts.states.find(), tree, 'states', tag_list=tag_list0)

	# List of initial states
	M = len(fts.states.initial)
	tag_list1 = [ 'state' for i in range(M) ]
	_export_list(fts.states.initial, tree, 'initial_states', tag_list=tag_list1)

	# List of actions
	if hasattr(fts, 'env_actions'):
		_export_list(fts.env_actions, tree, 'env_actions')
	if hasattr(fts, 'sys_actions'):
		_export_list(fts.sys_actions, tree, 'sys_actions')

	# List of transitions with actions. We're not going to use the usual export
	# list function to make things more clear
	transitions_list_node = ET.SubElement(tree, 'transitions')
	for transition in fts.transitions.find():
		transition_node = ET.SubElement(transitions_list_node, 'transition')
		state1 = transition[0]
		state2 = transition[1]
		actions_dict = transition[2]
		_export_xml(state1, transition_node, 'start_state')
		_export_xml(state2, transition_node, 'end_state')
#		start_node = ET.SubElement(transition_node, 'start_state')
#		start_node.text = state1
#		end_node = ET.SubElement(transition_node, 'end_state')
#		end_node.text = state2
		if 'sys_actions' in actions_dict.keys():
			_export_xml(actions_dict['sys_actions'], transition_node,
 				tag='sys_action')
		if 'env_actions' in actions_dict.keys():
			_export_xml(actions_dict['env_actions'], transition_node,
				tag='env_action')


#	_export_list(fts.transitions.find(), tree, 'transitions')
#	label_list = ['

	if parent is None:
		return tree



def _export_ppp(ppp, parent, tag):
	"""
	@return: None (if parent is None), or an xml tree
	@rtype: None or xml.etree.ElementTree.Element or
		xml.etree.ElementTree.SubElement
	"""
	if tag is None:
		tag = "PropPreservingPartition"

	if parent is None:
		tree = ET.Element(tag, type=T_PPP)
	else:
		tree = ET.SubElement(parent, tag, type=T_PPP)

	# Domain (polytope)
	_export_polytope(ppp.domain, tree, tag=N_DOMAIN)

	# regions (list of regions)
	_export_xml(ppp.regions, tree, N_REGIONLIST)

	# adj (adjacency matrix)
	_export_adj(ppp.adj, tree, N_ADJ)

	# prop regions (dictionary mapping strings to regions)
	_export_xml(ppp.prop_regions, tree, N_PROPREG)

	if parent is None:
		return tree


def _export_ltisys(ltisys, parent, tag=None):
	"""
	@return: None (if parent is None), or an xml tree
	@rtype: None or xml.etree.ElementTree.Element or
		xml.etree.ElementTree.SubElement
	"""
	if tag is None:
		tag = "LtiSysDyn"
	if parent is None:
		tree = ET.Element(tag, type=T_LTISYS)
	else:
		tree = ET.SubElement(parent, tag, type=T_LTISYS)

	# State space matrices
	_export_xml(ltisys.A, tree, 'A')
	_export_xml(ltisys.B, tree, 'B')
	_export_xml(ltisys.E, tree, 'E')
	_export_xml(ltisys.K, tree, 'K')

	# Domain matrices
	if ltisys.Uset is not None:
		_export_polytope(ltisys.Uset, tree, 'Uset')
	if ltisys.Wset is not None:
		_export_polytope(ltisys.Wset, tree, 'Wset')
	if ltisys.domain is not None:
		_export_polytope(ltisys.domain, tree, 'domain')

	if parent is None:
		return tree



def _export_pwasys(pwasys, parent, tag=None):
	"""
	@return: None (if parent is None), or an xml tree
	@rtype: None or xml.etree.ElementTree.Element or
		xml.etree.ElementTree.SubElement
	"""
	if tag is None:
		tag = "PwaSysDyn"
	if parent is None:
		tree = ET.Element(tag, type=T_PWASYS)
	else:
		tree = ET.SubElement(parent, tag, type=T_PWASYS)

	# Export domain
	if pwasys.domain is not None:
		_export_polytope(pwasys.domain, tree, 'domain')

	# Export lti list
	_export_list(pwasys.list_subsys, tree, 'ltilist')

	if parent is None:
		return tree


def _export_hybridsys(hybridsys, parent, tag=None):
	"""
	@return: None (if parent is None), or an xml tree
	@rtype: None or xml.etree.ElementTree.Element or
		xml.etree.ElementTree.SubElement
	"""
	if tag is None:
		tag = "SwitchedSysDyn"
	if parent is None:
		tree = ET.Element(tag, type=T_HYBRIDSYS)
	else:
		tree = ET.SubElement(parent, tag, type=T_HYBRIDSYS)

	# cts_ss
	_export_xml(hybridsys.cts_ss, tree, tag="cts_ss")

	# disc_domain_size
	_export_xml(hybridsys.disc_domain_size, tree, tag="disc_domain_size")

	# disc_sys_labels
	_export_xml(hybridsys.disc_sys_labels, tree, tag="sys_labels")

	# env_labels
	_export_xml(hybridsys.env_labels, tree, tag="env_labels")

	# Dynamics
	_export_dict(hybridsys.dynamics, tree, tag="dynamics")

	if parent is None:
		return tree



def _export_polytope(poly, parent, tag=None):
	"""Builds an XML tree from a polytope

	@param poly: Polytope to export
	@type poly: Polytope

	@return: None (if parent is None), or an xml tree
	@rtype: None or xml.etree.ElementTree.Element or
		xml.etree.ElementTree.SubElement
	"""

	if tag is None:
		tag = "polytope"

	if parent is None:
		tree = ET.Element(tag, type=T_POLYTOPE)
	else:
		tree = ET.SubElement(parent, tag, type=T_POLYTOPE)

	# A and b matrices
	_export_xml(poly.A, tree, 'A')
	_export_xml(poly.b, tree, 'b')


	# Optional parts

	# minimal representation (bool)
	if poly.minrep is not None:
		_export_xml(poly.minrep, tree, N_MINREP)

	# bounding box
	if poly.bbox is not None:
		bbox_node = ET.SubElement(tree, N_BBOX)
		_export_xml(poly.bbox[0], bbox_node, N_LOWERBOUND)
		_export_xml(poly.bbox[1], bbox_node, N_UPPERBOUND)

	# chebyshev center (ndarray)
	if poly.chebXc is not None:
		_export_xml(poly.chebXc, tree, N_CHEBXC)

	# chebyshev radius (float)
	if poly.chebR is not None:
		_export_xml(poly.chebR, tree, N_CHEBR)

	# dimension (integer)
	if poly.dim:
		_export_xml(poly.dim, tree, N_DIMENSION)

	# full dimension (bool)
	if poly.fulldim is not None:
		_export_xml(poly.fulldim, tree, N_FULLDIM)

	# volume (float)
	if poly.volume is not None:
		_export_xml(poly.volume, tree, N_VOLUME)

	# Return if there is no parent
	if parent is None:
		return tree



def _export_region(reg, parent, tag=None):
	"""
	@return: None (if parent is None), or an xml tree
	@rtype: None or xml.etree.ElementTree.Element or
		xml.etree.ElementTree.SubElement
	"""
	if tag is None:
		tag = "region"

	if parent is None:
		tree = ET.Element(tag, type=T_REGION)
	else:
		tree = ET.SubElement(parent, tag, type=T_REGION)

	# Attach list of polytopes
	_export_list(reg.list_poly, tree, N_POLYLIST)


	# Attach optional parts of region:

	# Bounding box, two numpy arrays
	if reg.bbox is not None:
		bbox_node = ET.SubElement(tree, N_BBOX)
		_export_xml(reg.bbox[0], bbox_node, N_LOWERBOUND)
		_export_xml(reg.bbox[1], bbox_node, N_UPPERBOUND)

	# Dimension (integer)
	if reg.dim:
		_export_xml(reg.dim, tree, N_DIMENSION)

	# Fulldim (bool)
	if reg.fulldim is not None:
		_export_xml(reg.fulldim, tree, N_FULLDIM)

	# Volume (float)
	if reg.volume is not None:
		_export_xml(reg.volume, tree, N_VOLUME)

	# Chebyshev radius (float)
	if reg.chebR is not None:
		_export_xml(reg.chebR, tree, N_CHEBR)

	# Chebyshev center (array)
	if reg.chebXc is not None:
		_export_xml(reg.chebXc, tree, N_CHEBXC)

	# Propositions that hold in region (set of strings)
	if reg.props:
		_export_xml(reg.props, tree, N_PROPS)

	if parent is None:
		return tree



def _export_adj(matrix, parent, tag=None):
	"""Converts an adjacency matrix (scipy.sparse.lil.lil_matrix) into an xml
	tree.

	@param matrix: Sparce adjacency matrix.
	@type matrix: scipy.sparse.lil.lil_matrix
	@type parent: None or xml.etree.ElementTree.Element or
		xml.etree.ElementTree.SubElement
	@type tag: string

	@return: None (if parent is None), or an xml tree
	@rtype: None or xml.etree.ElementTree.Element or
		xml.etree.ElementTree.SubElement
	"""

	if tag is None:
		tag = "adj"
	if parent is None:
		tree = ET.Element(tag, type=T_ADJ)
	else:
		tree = ET.SubElement(parent, tag, type=T_ADJ)

	# number of states, because the matrix must be square
	(M,N) = matrix.shape
	_export_xml(N, tree, "num_states")

	# list of nonzero indices
	(row_indices, col_indices) = matrix.nonzero()
	indices = []
	for i, row_ind in enumerate(row_indices):
		col_ind = col_indices[i]
		indices.append((row_ind, col_ind))
	_export_list(indices, tree, "index_list")

	if parent is None:
		return tree


def _export_dict(dictionary, parent, tag=None):
	"""
	Converts a dictionary into an XML tree. The key and value can be any
	supported type because the function calls _export_xml() on the key and
	value to form the document tree.

	@type dictionary: dict
	@type parent: None or xml.etree.ElementTree.Element or
		xml.etree.ElementTree.SubElement
	@type tag: string

	@return: An XML tree if parent is None. Otherwise, modifies the tree parent
		is contained in without returning.
	@rtype: xml.etree.ElementTree.Element or None
	"""

	if tag is None:
		tag = "dict"
	if parent is None:
		tree = ET.Element(tag, type=T_DICT)
	else:
		tree = ET.SubElement(parent, tag, type=T_DICT)

	# Make key value pairs
	for key, value in dictionary.iteritems():
		pair_node = ET.SubElement(tree, N_KEYVALUEPAIR)
		_export_xml(key, parent=pair_node, tag=N_KEY)
		_export_xml(value, parent=pair_node, tag=N_VALUE)

	if parent is None:
		return tree


def _export_list(lst, parent, tag=None, type_str=T_LIST, tag_list=[]):
	# Tag list is either a list of labels or a list of lists


	if tag is None:
		tag = "list"
	if parent is None:
		tree = ET.Element(tag, type=type_str)
	else:
		tree = ET.SubElement(parent, tag, type=type_str)

	if (tag_list and (len(tag_list) != len(lst))):
		raise ValueError('len(tag_list) != len(lst).')
	elif not tag_list:
		tag_list = [ N_ITEM for item in lst ]

	for ind, item in enumerate(lst):
		_export_xml(item, parent=tree, tag=tag_list[ind])

	if parent is None:
		return tree



"""
Export the controller synthesized by TuLiP

- only support Deterministic Mealy machine, with 1 init state
- only support controller synthesized by gr1c, NOT jtlv!!
"""
import numpy


def mealyexport(ctrl, filename="mealydata"):

    def input2num(sublabel_dict, inputs):
        result = 0
        tmp = 1
        for inputname, inputset in inputs.items():
            inputset = list(inputset)
            valuation = sublabel_dict[inputname]
            idx = inputset.index(valuation)
            result = result + idx * tmp
            tmp = tmp * len(inputset)

        return result

    fh = open('include/'+filename+".h", 'w')
    fc = open('sources/'+filename+".c", 'w')

    fh.write('''\
#ifndef __MEALY_H__
#define __MEALY_H__\n''')
    fh.write('#include <ecos.h>\n')
    fc.write('#include "'+filename+'.h"\n')

    inputs = ctrl.inputs

    if inputs is None:
        raise NotImplementedError

    fh.write('extern idxint nInputVariable;\n')
    fh.write('extern idxint nInputValue[];\n')

    numinputs = 1

    index = 0
    fc.write('idxint nInputVariable = ' + str(len(inputs.items())) + ';\n')
    fc.write('idxint nInputValue[] = {')
    for inputname, inputset in inputs.items():
        fc.write(str(len(inputset))+",")
        numinputs = numinputs * len(inputset)
        index = index + 1

    fc.write('};\n')

    fh.write("#define NUM_INPUT "+str(numinputs)+"\n")

    initialstate = ctrl.states.initial.pop()

    numstates = len(ctrl.states())

    fh.write("#define NUM_STATE "+str(numstates)+"\n")

    fh.write("extern int initState;\n")
    fh.write("extern int transition[NUM_STATE][NUM_INPUT];\n")
    fh.write("extern int output[NUM_STATE][NUM_INPUT];\n")

    statelist = []
    # create a list of states
    for state in ctrl.states():
        statelist.append(state)

    fc.write("int initState = "+str(statelist.index(initialstate))+";\n")

    # create the transition and output table:
    transition = numpy.zeros((numstates, numinputs), dtype='intc')
    output = numpy.zeros((numstates, numinputs), dtype='intc')
    for state in ctrl.states():
        index = statelist.index(state)
        trans = ctrl.transitions.find([state])
        for (from_state_, to_state, sublabel_dict) in trans:
            inputinedx = input2num(sublabel_dict, inputs)
            transition[index][inputinedx] = statelist.index(to_state)
            output[index][inputinedx] = sublabel_dict['loc']

    fc.write("int transition[NUM_STATE][NUM_INPUT]={\n")
    for i in range(0, numstates):
        fc.write("{")
        for j in range(0, numinputs):
            fc.write(str(transition[i][j])+",")
        fc.write("},\n")
    fc.write("};\n\n")

    fc.write("int output[NUM_STATE][NUM_INPUT]={\n")
    for i in range(0, numstates):
        fc.write("{")
        for j in range(0, numinputs):
            fc.write(str(output[i][j])+",")
        fc.write("},\n")
    fc.write("};\n\n")

    fh.write('int value2index(idxint inputValue[]);\n')

    fc.write('int value2index(idxint inputValue[])\n{\n')
    fc.write('	idxint result = 0;\n')
    fc.write('	int tmp = 1;\n')
    fc.write('	idxint valuation;\n')
    fc.write('	int i;\n')
    fc.write('	for(i=0;i<nInputVariable;i++)\n')
    fc.write('	{\n')
    fc.write('		valuation = inputValue[i]%nInputValue[i];\n')
    fc.write('		result += valuation * tmp;\n')
    fc.write('		tmp *= nInputValue[i];\n')
    fc.write('	}\n')
    fc.write('	return result;\n')
    fc.write('}\n')

    fh.write("#endif")
    fc.close()
    fh.close()



""" An aircraft electric system specification generator presented in the HSCC 2013 paper.

Huan Xu (mumu@caltech.edu)
October 30, 2012
"""

import sys, os
import re, copy
import numpy as np
import scipy
import scipy.io
import networkx as nx
import itertools
import time

#**************************************************************************************************************************************
def write_envgen(genlist):
    """Declares generator environment variable

    Parameters
    ----------
    genlist : list of all generators

    """
    for i in genlist:
        f.write('env_vars['"'"'g'+str(i)+"'"+'] = [0,1]')
        f.write('\n')
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def write_envru(rulist):
    """Declares rectifier unit environment variable

    Parameters
    ----------
    rulist : list of all rectifier units

    """
    for i in rulist:
        f.write('env_vars['"'"'ru'+str(i)+"'"+'] = [0,1]')
        f.write('\n')
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def write_essbusspec(essbuslist,time):
    """Writes counter for essential buses

    Parameters
    ----------
    essbuslist : list of buses with essential loads

    time: int
       max time bus can be unpowered

    """
    for i in essbuslist:
        f.write('guarantees += '"'"'&\\n\\t[]((b'+str(i)+'=0) -> (next(countb'+str(i)+') = countb'+str(i)+'+1))'"'")
        f.write('\n')
        f.write('guarantees += '"'"'&\\n\\t[]((b'+str(i)+'=1) -> (next(countb'+str(i)+') = 0))'"'")
        f.write('\n')
        f.write('guarantees += '"'"'&\\n\\t[](countb'+str(i)+' <= '+str(time)+')'"'")
        f.write('\n')
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def write_essbusdisc(essbuslist,time):
    """Declares bus counter system variable

    Parameters
    ----------
    essbuslist : list of all essential buses

    time: int
       max time bus can be unpowered

    """
    for i in essbuslist:
        f.write('disc_sys_vars['"'"'countb'+str(i)+"'"+'] = [x for x in range(0,'+str(time+1)+')]')
        f.write('\n')

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def write_discbus(buslist):
    """Declares bus system variables

    Parameters
    ----------
    buslist : list of all buses

    """
    for i in buslist:
        f.write('disc_sys_vars['"'"'b'+str(i)+"'"+'] = [0,1]')
        f.write('\n')
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def write_discnull(nullist):
    """Declares null node system variables

    Parameters
    ----------
    nullist : list of all null nodes

    """
    for i in nullist:
        f.write('disc_sys_vars['"'"'b'+str(i)+"'"+'] = [1]')
        f.write('\n')
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def write_discdc_con(G,rulist,dcbuslist,nullist):
    """Declares contactors (removes contactors between rus and dcbuses)

    Parameters
    ----------
    G : networkX graph

    rulist: list of rectifier units

    dcbuslist: list of all dc buses

    """
    remove = []
    remove2 = []
    for i in rulist:
        for j in dcbuslist:
            remove.append((i,j))
    L = copy.deepcopy(G)
    L.remove_edges_from(remove)

    remove2 = all_pairs(nullist)
    L.remove_edges_from(remove2)
    edges = L.edges()
    # print edges
    for i in range(0,len(edges)):
        # print edges[i][0]
        # print edges[i][1]
        f.write('disc_sys_vars['"'"'c'+str(edges[i][0])+str(edges[i][1])+"'"'] = [0,1]\n')
    for j in range(0,len(remove2)):
        f.write('disc_sys_vars['"'"'c'+str(remove2[j][0])+str(remove2[j][1])+"'"'] = [1]\n')
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def write_discac_con(G,nullist):
    """Declares contactors (removes contactors between rus and dcbuses)

    Parameters
    ----------
    G : networkX graph

    rulist: list of rectifier units

    dcbuslist: list of all dc buses

    """
    remove = []
    H = copy.deepcopy(G)
    if len(nullist) >= 2:
        remove = all_pairs(nullist)
        H.remove_edges_from(remove)
    edges = H.edges()
    for i in range(0,len(edges)):
        if edges[i][0] < edges[i][1]:
            f.write('disc_sys_vars['"'"'c'+str(edges[i][0])+str(edges[i][1])+"'"'] = [0,1]\n')
        else:
            f.write('disc_sys_vars['"'"'c'+str(edges[i][1])+str(edges[i][0])+"'"'] = [0,1]\n')
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def g_disconnect(G,genlist,buslist):
    """Writes specification disconnecting contactor if generator is unhealthy

    Parameters
    ----------
    G : networkX graph

    genlist : list of all generators

    buslist: list of all ac buses

    """
    for i in genlist:
        for j in buslist:
            for e in G.edges():
                if i in e and j in e:
                    if i < j:
                        f.write('guarantees += '"'"'&\\n\\t[]((g'+str(i)+'=0) -> (c'+str(i)+str(j)+'=0))'"'")
                    else:
                        f.write('guarantees += '"'"'&\\n\\t[]((g'+str(i)+'=0) -> (c'+str(j)+str(i)+'=0))'"'")
                    f.write('\n')
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def ru_disconnect(G,rulist,buslist):
    """Writes specification disconnecting contactor if rectifier is unhealthy

    Parameters
    ----------
    G : networkX graph

    rulist : list of all rectifiers

    buslist: list of all ac buses

    """
    for i in rulist:
        for j in buslist:
            for e in G.edges():
                if i in e and j in e:
                    if i < j:
                        f.write('guarantees += '"'"'&\\n\\t[]((ru'+str(i)+'=0) -> (c'+str(i)+str(j)+'=0))'"'")
                    else:
                        f.write('guarantees += '"'"'&\\n\\t[]((ru'+str(i)+'=0) -> (c'+str(j)+str(i)+'=0))'"'")
                    f.write('\n')
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def all_pairs(gens):
    """Returns list of all generator pairs (potential parallels).

    Parameters
    ----------
    gens : list
       list of all generator nodes

    """
    answer = []
    for i in range(len(gens)):
        for j in range(i+1, len(gens)):
            if (gens[i],gens[j]) not in answer:
                answer.append((gens[i],gens[j]))
    return answer
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def all_gens(list,G):
    """Finds all generator pairs that are connected through graph.

    Parameters
    ----------
    G : NetworkX graph

    list : tuples
       list of all generator pairs

    """
    pgens = []
    for i in range(len(list)):
        if nx.has_path(G,list[i][0], list[i][1]) is True:
            pgens.append(list[i])
    return pgens
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def ppaths(i,j,G):
    """Finds all contactors between two parallel sources. Converts to LTL.

    Parameters
    ----------
    G : NetworkX graph

    source i : node
       Starting node for path

    target j : node
       Ending node for path
    """
    result = nx.shortest_path(G,source=i,target=j)
    C = {}
    guarantees = 'guarantees += '"'"'&\\n\\t[](!('
    for k in range(0,len(result)-1):
        if result[k] < result[k+1]:
            C[k] = 'c'+str(result[k])+str(result[k+1])
        else:
            C[k] = 'c'+str(result[k+1])+str(result[k])
    guarantees = guarantees + '('+str(C[0])+'=1)'
    for m in range(1,len(C)):
        guarantees = guarantees+ ' & ('+str(C[m])+'=1)'
    guarantees = guarantees + '))'"'"
    return guarantees
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def noparallel(G,list):
    """Writes non-paralleling specifications to file.

    Parameters
    ----------
    G : NetworkX graph

    list : tuples
       list of all generator pairs with paths in G

    """
    sourcetemp = (all_pairs(list))
    source = all_gens(sourcetemp,G)
    for i in range(0,len(source)):
        mat = ppaths(source[i][0], source[i][1], G)
        f.write(mat)
        f.write('\n')
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def faulttol(prob,allgens,genfail):
    """Finds all combinations of failures.

    Parameters
    ----------
    prob: int
       probability up to which failure can occur

    allgens: list
       list of all components that can fail

    genfail: int
       probability of failure of single component

    """
    tuples = int(prob/genfail)
    fails = []
    temp = []
    if tuples <= 1:
        fails = allgens[:]
    else:
        fails = allgens[:]
        for i in range(2,tuples+1):
            for temp in itertools.combinations(allgens,i):
                fails.append(temp)
    return fails
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def write_genassump(genfail,genlist):
    """Writes generator environment assumption

    Parameters
    ----------
    genfail : int
       how many generators may fail at once time

    genlist : list of all generators

    """
    f.write('assumptions += '"'"'&\\n\\t[]((g'+str(genlist[0]))
    for i in range(1,len(genlist)):
        f.write(' + g'+str(genlist[i]))
    f.write(') >= '+str(len(genlist)-genfail)+')'"'")
    f.write('\n')
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def write_ruassump(rufail,rulist):
    """Writes rectifier environment assumption

    Parameters
    ----------
    rufail : int
       how many rectifiers may fail at once time

    genlist : list of all generators

    """
    if len(rulist) > 0:
        f.write('assumptions += '"'"'&\\n\\t[]((ru'+str(rulist[0]))
        for i in range(1,len(rulist)):
            f.write(' + ru'+str(rulist[i]))
        f.write(') >= '+str(len(rulist)-rufail)+')'"'")
        f.write('\n')
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def remove_ru_edges(G,buslist,rulist):
    pairs = []
    H = copy.deepcopy(G)
    if len(buslist) > 0:
        for i in busac:
            for j in rus:
                pairs.append((i,j))
    return pairs
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def remove_rus(G,buslist,rulist):
    pairs = []
    H = copy.deepcopy(G)
    if len(buslist) > 0:
        for i in busac:
            for j in rus:
                pairs.append((i,j))
        H.remove_edges_from(pairs)
    return H
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def buspathnodes(G,busno,source):
    """Finds determines if path exists from bus to generator.

    Parameters
    ----------
    busno: int
       node number for bus

    sources: list
       list of all generators

    G: NetworkX graph

    """
    buspaths = []
    if nx.has_path(G,busno,source):
            buspaths.append((busno,source))
    return buspaths
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def acbusprop(G,source,target):
    paths = []
    C = []
    temp = []
    edges = []
    D = copy.deepcopy(G)
    gens2 = copy.deepcopy(gens)
    gens2.remove(target)
    for m in gens2:
        temp.append((source,m))
    D.remove_edges_from(temp)
    edges = remove_ru_edges(G,busac,rus)
    D.remove_edges_from(edges)
    for path in nx.all_simple_paths(D,source,target):
        paths.append(path)
        for p in range(0,len(paths)):
            for i in range(0,len(paths[p])-1):
                C.append((paths[p][i],paths[p][i+1]))
            f.write('disc_props['"'"'B'+str(source)+str(target)+str(p)+"'"'] = '"'")
            if paths[p][1] in gens:
                f.write('(g'+str(paths[p][1])+'=1)')
            elif paths[p][1] in busac:
                f.write('(b'+str(paths[p][1])+'=1)')
            elif paths[p][1] in null:
                f.write('(b'+str(paths[p][1])+'=1)')
            else:
                pass
            if len(paths[p]) > 2:
                for j in range(2,len(paths[p])):
                    if paths[p][j] in gens:
                        f.write(' & (g'+str(paths[p][j])+'=1)')
                    elif paths[p][j] in busac:
                        f.write(' & (b'+str(paths[p][j])+'=1)')
                    elif paths[p][j] in null:
                        f.write(' & (b'+str(paths[p][j])+'=1)')
                    else:
                        pass
            for k in range(0,len(C)):
                if C[k][0] < C[k][1]:
                    f.write(' & (c'+str(C[k][0])+str(C[k][1])+'=1)')
                else:
                    f.write(' & (c'+str(C[k][1])+str(C[k][0])+'=1)')
            f.write("'"'\n')
            C = []
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def write_acbusprop(G,buslist,genlist):
    """Writes dc bus properties

    Parameters
    ----------
    G : networkX graph

    buslist : list of all dc buses

    genlist : list of all generators

    """
    for i in buslist:
        for j in genlist:
            acbusprop(G,i,j)
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def acbusspec(G,busno,gen):
    temp = []
    edges = []
    D = copy.deepcopy(G)
    gens2 = copy.deepcopy(gens)
    gens2.remove(gen)
    for m in gens2:
        temp.append((busno,m))
    D.remove_edges_from(temp)
    edges = remove_ru_edges(G,busac,rus)
    D.remove_edges_from(edges)
    paths = []
    for path in nx.all_simple_paths(D,busno,gen,cutoff=None):
        paths.append(path)
    for j in range(0,len(paths)):
        f.write('guarantees += '"'"'&\\n\\t[]((B'+str(busno)+str(gen)+str(j)+') -> (b'+str(busno)+'=1))'"'"'\n')
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def write_acbusspec(G,buslist,genlist):
    """Writes dc bus specifications

    Parameters
    ----------
    G : networkX graph

    buslist : list of all dc buses

    genlist : list of all generators

    """
    for i in buslist:
        for j in genlist:
            acbusspec(G,i,j)
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def write_acbusspec2(G,buslist,genlist):
    """Writes specifications for dc bus unpowered conditions

    Parameters
    ----------
    G : networkX graph

    buslist : list of all dc buses

    genlist : list of all generators

    """
    paths = []
    temp = []
    edges = []
    D = copy.deepcopy(G)
    gens2 = copy.deepcopy(gens)
    edges = remove_ru_edges(G,busac,rus)
    D.remove_edges_from(edges)
    for i in buslist:
        f.write('guarantees += '"'"'&\\n\\t[](!((0=1)')
        for j in genlist:
            gens2.remove(j)
            D.remove_nodes_from(gens2)
            for path in nx.all_simple_paths(D,i,j):
                paths.append(path)
                f.write(' | (B' + str(i) + str(j) + str(len(paths)-1)+')')
            paths = []
            gens2 = copy.deepcopy(gens)
            D = copy.deepcopy(G)
        f.write(') -> (b'+str(i)+'=0))'"'"'\n')
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def write_dcbusalways(buslist):
    """Writes dc bus specification must always be powered

    Parameters
    ----------
    buslist : list of all dc buses

    """
    for i in buslist:
        f.write('guarantees += '"'"'&\\n\\t[](b'+str(i)+' = 1)'"'"+'\n')
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def dcbusprop(G,source,target):
    """Creates discrete properties for power status of dc buses

    Parameters
    ----------
    G : networkX graph

    source : node
       dc bus

    target : node
       generator

    """
    temp = []
    edges = []
    D = copy.deepcopy(G)
    gens2 = copy.deepcopy(gens)
    gens2.remove(target)
    D.remove_nodes_from(gens2)
    paths = []
    C = []
    for path in nx.all_simple_paths(D,source,target,cutoff=None):
        paths.append(path)
    for p in range(0,len(paths)):
        for i in range(0,len(paths[p])-1):
            if paths[p][i] in busdc and paths[p][i+1] in rus:
                pass
            elif paths[p][i] in rus and paths[p][i+1] in busdc:
                pass
            else:
                C.append((paths[p][i],paths[p][i+1]))
        f.write('disc_props['"'"'B'+str(source)+str(target)+str(p)+"'"'] = '"'")
        if paths[p][1] in gens:
            f.write('(g'+str(paths[p][1])+'=1)')
        elif paths[p][1] in busac:
            f.write('(b'+str(paths[p][1])+'=1)')
        elif paths[p][1] in rus:
            f.write('(ru'+str(paths[p][1])+'=1)')
        elif paths[p][1] in busdc:
            f.write('(b'+str(paths[p][1])+'=1)')
        elif paths[p][1] in null:
            f.write('(b'+str(paths[p][1])+'=1)')
        else:
            pass
        for j in range(2,len(paths[p])):
            if paths[p][j] in gens:
                f.write(' & (g'+str(paths[p][j])+'=1)')
            elif paths[p][j] in busac:
                f.write(' & (b'+str(paths[p][j])+'=1)')
            elif paths[p][j] in rus:
                f.write(' & (ru'+str(paths[p][j])+'=1)')
            elif paths[p][1] in busdc:
                f.write(' & (b'+str(paths[p][1])+'=1)')
            elif paths[p][1] in null:
                f.write(' & (b'+str(paths[p][1])+'=1)')
            else:
                pass
        for k in range(0,len(C)):
            if C[k][0] < C[k][1]:
                f.write(' & (c'+str(C[k][0])+str(C[k][1])+'=1)')
            else:
                f.write(' & (c'+str(C[k][1])+str(C[k][0])+'=1)')
        f.write("'"'\n')
        C = []

def write_dcbusprop(G,buslist,genlist):
    """Writes dc bus properties

    Parameters
    ----------
    G : networkX graph

    buslist : list of all dc buses

    genlist : list of all generators

    """
    for i in buslist:
        for j in genlist:
            dcbusprop(G,i,j)

def dcbusspec(G,busno,gen):
    """Creates specifications for when DC bus gets powered

    Parameters
    ----------
    G : networkX graph

    busno : node
       dc bus

    gen : node
       generator

    """
    paths = []
    C = []
    temp = []
    edges = []
    D = copy.deepcopy(G)
    gens2 = copy.deepcopy(gens)
    gens2.remove(gen)
    D.remove_nodes_from(gens2)
    for path in nx.all_simple_paths(D,busno,gen,cutoff=None):
        paths.append(path)
    for j in range(0,len(paths)):
        f.write('guarantees += '"'"'&\\n\\t[]((B'+str(busno)+str(gen)+str(j)+') -> (b'+str(busno)+'=1))'"'"'\n')

def write_dcbusspec(G,buslist,genlist):
    """Writes dc bus specifications

    Parameters
    ----------
    G : networkX graph

    buslist : list of all dc buses

    genlist : list of all generators

    """
    paths = []
    for i in buslist:
        for j in genlist:
            dcbusspec(G,i,j)

def write_dcbusspec2(G,buslist,genlist):
    """Writes specifications for dc bus unpowered conditions

    Parameters
    ----------
    G : networkX graph

    buslist : list of all dc buses

    genlist : list of all generators

    """
    paths = []
    temp = []
    edges = []
    D = copy.deepcopy(G)
    gens2 = copy.deepcopy(gens)
    for i in buslist:
        f.write('guarantees += '"'"'&\\n\\t[](!((0=1)')
        for j in genlist:
            gens2.remove(j)
            D.remove_nodes_from(gens2)
            for path in nx.all_simple_paths(D,i,j):
                paths.append(path)
                f.write(' | (B' + str(i) + str(j) + str(len(paths)-1)+')')
            paths = []
            gens2 = copy.deepcopy(gens)
            D = copy.deepcopy(G)
        f.write(') -> (b'+str(i)+'=0))'"'"'\n')
#************************************************************************************************
def write_sat_bool(complist):
    """Defines boolean components (not including contactors)

    Parameters
    ----------
    complist : list of all components

    """
    for i in complist:
        if i in gens:
            f.write('(define g'+str(i)+'::bool)\n')
        elif i in busac:
            f.write('(define b'+str(i)+'::bool)\n')
        elif i in busdc:
            f.write('(define b'+str(i)+'::bool)\n')
        elif i in null:
            f.write('(define b'+str(i)+'::bool)\n')
        elif i in rus:
            f.write('(define r'+str(i)+'::bool)\n')
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def write_sat_con(G,rulist,dcbuslist,nullist):
    """Defines contactors (removes contactors between rus and dcbuses)

    Parameters
    ----------
    G : networkX graph

    rulist: list of rectifier units

    dcbuslist: list of all dc buses

    """
    remove = []
    remove2 = []
    for i in rulist:
        for j in dcbuslist:
            remove.append((i,j))
    L = copy.deepcopy(G)
    L.remove_edges_from(remove)

    remove2 = all_pairs(nullist)
    L.remove_edges_from(remove2)
    edges = L.edges()
    for i in range(0,len(edges)):
        f.write('(define c'+str(edges[i][0])+str(edges[i][1])+'::bool)\n')
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def write_sat_always(complist):
    """Asserts boolean components always on/powered

    Parameters
    ----------
    complist : list of all components

    """
    remove2 = []
    for i in complist:
        if i in busac:
            f.write('(assert (= b'+str(i)+' true))\n')
        elif i in busdc:
            f.write('(assert (= b'+str(i)+' true))\n')
        elif i in null:
            f.write('(assert (= b'+str(i)+' true))\n')

    remove2 = all_pairs(null)
    edges = G.edges()
    for j in range(0,len(remove2)):
        if remove2[j] in edges:
            f.write('(assert (= c'+str(remove2[j][0])+str(remove2[j][1])+' true))\n')
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def write_sat_disconnectgen(G,genlist):
    """Writes specification disconnecting contactor if generator is unhealthy

    Parameters
    ----------
    G : networkX graph

    genlist : list of all generators
    """
    neighbor = []
    for i in genlist:
        neighbor = G.neighbors(i)
        for j in range(0, len(neighbor)):
            if i < neighbor[j]:
                f.write('(assert (=> (= g'+str(i)+' false) (= c'+str(i)+str(neighbor[j])+' false)))\n')
            elif i > neighbor[j]:
                f.write('(assert (=> (= g'+str(i)+' false) (= c'+str(neighbor[j])+str(i)+' false)))\n')
        neighbor = []
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def write_sat_disconnectru(G,rulist):
    """Writes specification disconnecting contactor if rectifier is unhealthy

    Parameters
    ----------
    G : networkX graph

    genlist : list of all generators
    """
    neighbor = []
    H = copy.deepcopy(G)
    H.remove_nodes_from(busdc)
    for i in rulist:
        neighbor = H.neighbors(i)
        for j in range(0, len(neighbor)):
            if i < neighbor[j]:
                f.write('(assert (=> (= r'+str(i)+' false) (= c'+str(i)+str(neighbor[j])+' false)))\n')
            elif i > neighbor[j]:
                f.write('(assert (=> (= r'+str(i)+' false) (= c'+str(neighbor[j])+str(i)+' false)))\n')
        neighbor = []
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def write_sat_noparallel(G,genlist):
    pairs = all_pairs(genlist)
    H = copy.deepcopy(G)
    H.remove_nodes_from(rus)
    for i in range(0, len(pairs)):
        for path in nx.all_simple_paths(H, pairs[i][0], pairs[i][1]):
            f.write('(assert (not (and ')
            for j in range(0,len(path)-1):
                if path[j] < path[j+1]:
                    f.write('(= c'+str(path[j])+str(path[j+1])+' true) ')
                else:
                    f.write('(= c'+str(path[j+1])+str(path[j])+' true) ')
            f.write(')))\n')
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def write_sat_acbusprop1(G,buslist, genlist):
    H = copy.deepcopy(G)
    H.remove_nodes_from(rus)
    for i in buslist:
        for j in genlist:
            gen_temp = copy.deepcopy(gens)
            gen_temp.remove(j)
            H.remove_nodes_from(gen_temp)
            H.remove_nodes_from(rus)
            for path in nx.all_simple_paths(H,i,j):
                f.write('(assert (=> (and ')
                for k in range(1,len(path)):
                    if path[k] in gens:
                        f.write(' (= g'+str(path[k])+' true)')
                    elif path[k] in busac:
                        f.write(' (= b'+str(path[k])+' true)')
                    elif path[k] in null:
                        f.write(' (= b'+str(path[k])+' true)')
                for m in range(0,len(path)-1):
                    if path[m] < path[m+1]:
                        f.write(' (= c'+str(path[m])+str(path[m+1])+' true)')
                    else:
                        f.write(' (= c'+str(path[m+1])+str(path[m])+' true)')
                f.write(') (= b'+str(i)+' true)))\n')
            H = copy.deepcopy(G)
            H.remove_nodes_from(rus)
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def write_sat_acbusprop2(G,buslist, genlist):
    H = copy.deepcopy(G)
    H.remove_nodes_from(rus)
    # i = buslist[7]
    # j = genlist[5]
    for i in buslist:
        f.write('(assert (=> (not (or ')
        for j in genlist:
            gen_temp = copy.deepcopy(gens)
            gen_temp.remove(j)
            H.remove_nodes_from(gen_temp)
            H.remove_nodes_from(rus)
            for path in nx.all_simple_paths(H,i,j):
                f.write(' (and')
                for k in range(1,len(path)):
                        if path[k] in gens:
                            f.write(' (= g'+str(path[k])+' true)')
                        elif path[k] in busac:
                            f.write(' (= b'+str(path[k])+' true)')
                        elif path[k] in null:
                            f.write(' (= b'+str(path[k])+' true)')
                for m in range(0,len(path)-1):
                    if path[m] < path[m+1]:
                        f.write(' (= c'+str(path[m])+str(path[m+1])+' true)')
                    else:
                        f.write(' (= c'+str(path[m+1])+str(path[m])+' true)')
                f.write(')')
            H = copy.deepcopy(G)
            H.remove_nodes_from(rus)
        f.write(')) (= b'+str(i)+ ' false)))\n')

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def write_sat_dcbusprop1(G,buslist, genlist):
    H = copy.deepcopy(G)
    for i in buslist:
        for j in genlist:
            gen_temp = copy.deepcopy(gens)
            gen_temp.remove(j)
            H.remove_nodes_from(gen_temp)
            for path in nx.all_simple_paths(H,i,j):
                f.write('(assert (=> (and ')
                for k in range(1,len(path)):
                    if path[k] in gens:
                        f.write(' (= g'+str(path[k])+' true)')
                    elif path[k] in busac:
                        f.write(' (= b'+str(path[k])+' true)')
                    elif path[k] in null:
                        f.write(' (= b'+str(path[k])+' true)')
                    elif path[k] in rus:
                        f.write(' (= r'+str(path[k])+' true)')
                for m in range(0,len(path)-1):
                    if path[m] in busdc and path[m+1] in rus:
                        pass
                    elif path[m] in rus and path[m+1] in busdc:
                        pass
                    elif path[m] < path[m+1]:
                        f.write(' (= c'+str(path[m])+str(path[m+1])+' true)')
                    elif path[m+1] < path[m]:
                        f.write(' (= c'+str(path[m+1])+str(path[m])+' true)')
                f.write(') (= b'+str(i)+' true)))\n')
            H = copy.deepcopy(G)
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def write_sat_dcbusprop2(G,buslist, genlist):
    H = copy.deepcopy(G)
    for i in buslist:
        f.write('(assert (=> (not (or ')
        for j in genlist:
            gen_temp = copy.deepcopy(gens)
            gen_temp.remove(j)
            H.remove_nodes_from(gen_temp)
            for path in nx.all_simple_paths(H,i,j):
                f.write(' (and')
                for k in range(1,len(path)):
                    if path[k] in gens:
                        f.write(' (= g'+str(path[k])+' true)')
                    elif path[k] in busac:
                        f.write(' (= b'+str(path[k])+' true)')
                    elif path[k] in null:
                        f.write(' (= b'+str(path[k])+' true)')
                    elif path[k] in rus:
                        f.write(' (= r'+str(path[k])+' true)')
                for m in range(0,len(path)-1):
                    if path[m] in busdc and path[m+1] in rus:
                        pass
                    elif path[m] in rus and path[m+1] in busdc:
                        pass
                    elif path[m] < path[m+1]:
                        f.write(' (= c'+str(path[m])+str(path[m+1])+' true)')
                    elif path[m+1] < path[m]:
                        f.write(' (= c'+str(path[m+1])+str(path[m])+' true)')
                f.write(')')
            H = copy.deepcopy(G)
        f.write(')) (= b'+str(i)+' false)))\n')
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def write_sat_env(gfail,rfail):
    gentemp = [x for x in range(0,gfail+1)]
    rutemp = [x for x in range(0,rfail+1)]
    allgens = []
    allrus = []
    env_filename = 'env'
    count = 0
    for i in gentemp:
        for j in itertools.combinations(gens,len(gens)-i):
            if len(gens)-i == 1:
                for k in gens:
                    allgens.append(k)
            else:
                allgens.append(j)
    allgens = list(set(allgens))
    if len(rus) == 0:
        pass
    else:
        for i in rutemp:
            for j in itertools.combinations(rus,len(rus)-i):
                if len(rus)-i == 1:
                    for k in rus:
                        allrus.append(k)
                else:
                    allrus.append(j)
    allrus = list(set(allrus))
    for i in range(0,len(allgens)):
        for j in range(0,len(allrus)):
            env_filename = 'env'+str(count)+'.ys'
            f2 = open(env_filename,"w")
            for k in gens:
                if isinstance(allgens[i],int):
                    if k == allgens[i]:
                        f2.write('(assert (= g'+str(k)+' true))\n')
                    else:
                        f2.write('(assert (= g'+str(k)+' false))\n')
                elif k in allgens[i]:
                    f2.write('(assert (= g'+str(k)+' true))\n')
                else:
                    f2.write('(assert (= g'+str(k)+' false))\n')
            for m in rus:
                if isinstance(allrus[j],int):
                    if m == allrus[j]:
                        f2.write('(assert (= r'+str(m)+' true))\n')
                    else:
                        f2.write('(assert (= r'+str(m)+' false))\n')
                elif m in allrus[j]:
                    f2.write('(assert (= r'+str(m)+' true))\n')
                else:
                    f2.write('(assert (= r'+str(m)+' false))\n')
            count = count+1
            f2.write('(check)\n')
            f2.close()

#************************************************************************************************
start = time.time()
file_name = 'test_spec'

#Load adjacency matrix from matfile
data = scipy.io.loadmat('SLD.mat')
datatemp = data['A']
# create matrix from matfile
A = np.matrix(datatemp)

#Failure Probabilities
genfail = 1
rufail = 1
busfail = 0

#Node definitions
busac = [2,3]
busess = [2,3]
busdc = [6,7]
null = []
rus = [4,5]
gens = [0,1]

#Bus time
nptime = 0

#Create networkx graph from adjacency matrix
G=nx.from_numpy_matrix(A)
print 'number of edges ' + str(len(G.edges()))
print 'number of nodes ' + str(len(G.nodes()))

#sets of failure states
# fails = faulttol(10,gens,genfail)

################################################################
# Synthesize
################################################################
if ('tulip' in sys.argv):
    file_name = file_name+'.py'
    f = open(file_name, "w")

    # #environment variables
    print 'writing environment variables'
    write_envgen(gens)
    write_envru(rus)

    #discrete system variables
    print 'writing discrete system variables: buses'
    write_discbus(busac)
    write_discbus(busdc)
    write_discnull(null)
    print time.time()-start
    print 'writing discrete system variables: contactors'
    # write_discac_con(G,null) #only use this when there are NO DC components
    write_discdc_con(G,rus,busdc,null)
    print time.time()-start
    print 'writing discrete system variables: bus counters'
    write_essbusdisc(busess,nptime)

    #acbus discrete properties
    print 'removing ru paths'
    H = remove_rus(G,busac,rus)
    print time.time()-start
    print 'writing discrete bus properties: AC'
    write_acbusprop(H,busac,gens)
    print time.time()-start
    print 'writing discrete bus properties: DC'
    write_dcbusprop(G,busdc,gens)
    print time.time()-start

    #Environment assumptions
    print 'writing environment assumptions'
    write_genassump(genfail,gens)
    write_ruassump(rufail,rus)
    print time.time()-start

    #acbus power guarantees
    print 'writing bus power specifications: AC'
    write_acbusspec(H,busac,gens)
    write_acbusspec2(H,busac,gens)
    write_essbusspec(busess,nptime)
    print time.time()-start

    #disconnect unhealthy guarantees
    print 'disconnecting unhealthy generators'
    g_disconnect(G,gens,busac)
    g_disconnect(G,gens,null)
    print time.time()-start
    print 'disconnecting unhealthy rus'
    ru_disconnect(G,rus,busac)
    ru_disconnect(G,rus,null)
    print time.time()-start

    ##non-parallel guarantees
    print 'writing no paralleling specs'
    noparallel(G,gens)
    print time.time()-start

    #dc bus power specifications
    print 'writing bus power specifications: DC'
    write_dcbusspec(G,busdc,gens)
    write_dcbusspec2(G,busdc,gens)
    write_dcbusalways(busdc)

    f.close()

    print 'It took', time.time()-start, 'seconds.'

################################################################
# Synthesize
################################################################
if ('yices' in sys.argv):
    file_name = file_name+'.ys'
    f = open(file_name, "w")

    #Writing component definitions
    write_sat_bool(gens)
    write_sat_bool(busac)
    write_sat_bool(busdc)
    write_sat_bool(null)
    write_sat_bool(rus)

    #Writing contactor definitions
    write_sat_con(G,rus,busdc,null)

    #Writing always power bus and null assertions
    write_sat_always(busac)
    write_sat_always(busdc)
    write_sat_always(null)

    #Writing disconnect implications
    write_sat_disconnectgen(G,gens)
    write_sat_disconnectru(G,rus)

    write_sat_noparallel(G,gens)
    write_sat_acbusprop1(G,busac,gens)
    write_sat_acbusprop2(G,busac,gens)
    write_sat_dcbusprop1(G,busdc, gens)
    write_sat_dcbusprop2(G,busdc, gens)

    #write_environment assumptions
    write_sat_env(genfail,rufail)

    f.close()
    print 'It took', time.time()-start, 'seconds.'



#!/usr/bin/env python
"""
Usage: solverand.py [H W]

will generate a random deterministic gridworld of height H and width W
(default is 5 by 10), try to solve it and, if realizable, create an
SVG file named "ctrl-solverand.svg" that depicts the controller, which
is a Mealy machine.  Example usage for 3 by 5 size is

  $ ./solverand.py 3 5
"""
from __future__ import print_function

import sys
import tulip.gridworld as gw
from tulip import synth


if len(sys.argv) > 3 or "-h" in sys.argv:
    print("Usage: solverand.py [H W]")
    sys.exit(1)

if len(sys.argv) >= 3:
    (height, width) = (int(sys.argv[1]), int(sys.argv[2]))
else:
    (height, width) = (5, 10)

Z = gw.random_world((height, width),
                    wall_density=0.2,
                    num_init=1,
                    num_goals=2)
print(Z)

spc = Z.spec()
spc.moore = False
spc.qinit = r'\A \E'
if not synth.is_realizable('omega', spc):
    print("Not realizable.")
else:
    ctrl = synth.synthesize('omega', spc)
    if not ctrl.save('ctrl-solverand.svg'):
        print(ctrl)



"""
Simple example with uncontrolled switching, for debugging

 6 cell robot example.
     +---+---+---+
     | 3 | 4 | 5 |
     +---+---+---+
     | 0 | 1 | 2 |
     +---+---+---+
"""
from tulip import spec, synth, transys
import numpy as np
from scipy import sparse as sp

sys_swe = transys.FTS()
sys_swe.env_actions.add_from({'sun', 'rain'})

# Environment actions are mutually exclusive.
n = 2
states = ['s' + str(i) for i in xrange(n) ]
sys_swe.states.add_from(states)
sys_swe.states.initial |= ['s0']

# different transitions possible, depending on weather
transmat1 = sp.lil_matrix(np.array(
                [[0,1],
                 [1,0]]
            ))
sys_swe.transitions.add_adj(transmat1, states, env_actions='sun')

# avoid being killed by environment
transmat2 = sp.lil_matrix(np.array(
                [[1,0],
                 [0,1]]
            ))
sys_swe.transitions.add_adj(transmat2, states, env_actions='rain')

# atomic props
sys_swe.atomic_propositions.add_from(['home', 'lot'])

sys_swe.states.add(states[0], ap={'home'})
sys_swe.states.add(states[1], ap={'lot'})

print(sys_swe)

sys_swe.save('sys_swe.pdf')

# (park & sun) & []<>!park && []<>sum
env_vars = {'park'}
env_init = {'park', 'env_actions = "sun"'}
env_prog = {'!park','env_actions = "sun"'}
env_safe = set()

# (s0 & mem) & []<> home & [](park -> <>lot)
sys_vars = {'mem'}
sys_init = {'mem'}
sys_prog = {'home'}               # []<>home
sys_safe = {'next(mem) <-> lot || (mem && !park)'}
sys_prog |= {'mem'}

# Create the specification
specs = spec.GRSpec(env_vars, sys_vars, env_init, sys_init,
                    env_safe, sys_safe, env_prog, sys_prog)
specs.moore = False
specs.qinit = '\A \E'
# Controller synthesis
ctrl = synth.synthesize('omega', specs, sys=sys_swe,
                        ignore_sys_init=True)

if not ctrl.save('switch.pdf'):
    print(ctrl)



#!/usr/bin/env python
"""Discrete synthesis from a dummy abstraction with uncontrolled switching.

This is an example to demonstrate how the output of the TuLiP discretization
for a system with uncontrollable switching (i.e., modes are controlled by the
environment) might look like.

We will assume, we have the 6 cell robot example.

     +---+---+---+
     | 3 | 4 | 5 |
     +---+---+---+
     | 0 | 1 | 2 |
     +---+---+---+
"""
# NO, 26 Jul 2013.
import numpy as np
from scipy import sparse as sp
from tulip import spec
from tulip import synth
from tulip import transys


###########################################
# Environment switched system with 2 modes:
###########################################

sys_swe = transys.FTS()

# We assume robots ability to transition between cells depends on the surface
# characteristics which could be slippery or normal. This is controlled by the
# environment.

sys_swe.env_actions.add_from({'slippery','normal'})
# Environment actions are mutually exclusive.

# Discretization builds a transition matrix (invisible to the end user)

n = 6
states = ['s'+str(i) for i in xrange(n) ]

sys_swe.atomic_propositions.add_from(['home','lot'])
state_labels = [{'home'}, set(), set(), set(), set(), {'lot'}]

# Add states and decorate TS with state labels (aka atomic propositions)
for state, label in zip(states, state_labels):
    sys_swe.states.add(state, ap=label)

# Within each mode the transitions can be deterministically chosen, environment
# chooses the mode (the surface can be slippery or normal).
transmat1 = sp.lil_matrix(np.array(
                [[1,1,0,1,0,0],
                 [1,1,1,0,1,0],
                 [0,1,1,0,1,1],
                 [1,0,0,1,1,0],
                 [0,1,0,1,1,1],
                 [0,0,1,0,1,1]]
            ))

sys_swe.transitions.add_adj(transmat1, states, env_actions='normal')

# In slippery mode, the robot can't stay still and makes larger jumps.
transmat2 = sp.lil_matrix(np.array(
                [[0,0,1,1,0,0],
                 [1,0,1,0,1,0],
                 [1,0,0,0,1,1],
                 [1,0,0,0,0,1],
                 [0,1,0,1,0,1],
                 [0,0,1,1,0,0]]
            ))

sys_swe.transitions.add_adj(transmat2, states, env_actions='slippery')

# This is what is visible to the outside world (and will go into synthesis method)
print(sys_swe)

#
# Environment variables and specification
#
# The environment can issue a park signal that the robot just respond
# to by moving to the lower left corner of the grid.  We assume that
# the park signal is turned off infinitely often.
#
env_vars = {'park'}
env_init = set()                # empty set
env_prog = {'!park'}
env_safe = set()                # empty set

#
# System specification
#
# The system specification is that the robot should repeatedly revisit
# the upper right corner of the grid while at the same time responding
# to the park signal by visiting the lower left corner.  The LTL
# specification is given by
#
#     []<> home && [](park -> <>lot)
#
# Since this specification is not in GR(1) form, we introduce the
# variable X0reach that is initialized to True and the specification
# [](park -> <>lot) becomes
#
#     [](X (X0reach) <-> lot || (X0reach && !park))
#

# Augment the environmental description to make it GR(1)
#! TODO: create a function to convert this type of spec automatically

# Define the specification
#! NOTE: maybe "synthesize" should infer the atomic proposition from the
# transition system? Or, we can declare the mode variable, and the values
# of the mode variable are read from the transition system.
sys_vars = {'X0reach'}
sys_init = {'X0reach'}
sys_prog = {'home'}               # []<>home
sys_safe = {'(X (X0reach) <-> lot) || (X0reach && !park)'}
sys_prog |= {'X0reach'}

# Create the specification
specs = spec.GRSpec(env_vars, sys_vars, env_init, sys_init,
                    env_safe, sys_safe, env_prog, sys_prog)
# controller decides based on current values `env_vars, sys_vars`
# and next values `env_vars'`. A controller with this
# information flow is known as Mealy.
specs.moore = False
specs.qinit = '\A \E'

# Controller synthesis
#
# At this point we can synthesize the controller using one of the available
# methods.
#
ctrl = synth.synthesize('omega', specs, sys=sys_swe, ignore_sys_init=True)
assert ctrl is not None, 'unrealizable'

# @plot_print@
if not ctrl.save('environment_switching.png'):
    print(ctrl)
# @plot_print_end@
